#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_switch_dhcp_server_policy_arp_inspection_trusted_servers
short_description: Resource module for networks _switch _dhcp _server _policy _arp
  _inspection _trusted _servers
description:
  - Manage operations create, update and delete of the resource networks _switch _dhcp
    _server _policy _arp _inspection _trusted _servers.
  - Add a server to be trusted by Dynamic ARP Inspection on this network.
  - Remove a server from being trusted by Dynamic ARP Inspection on this network.
  - Update a server that is trusted by Dynamic ARP Inspection on this network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  ipv4:
    description: The IPv4 attributes of the trusted server being added.
    suboptions:
      address:
        description: The IPv4 address of the trusted server being added.
        type: str
    type: dict
  mac:
    description: The mac address of the trusted server being added.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  trustedServerId:
    description: TrustedServerId path parameter. Trusted server ID.
    type: str
  vlan:
    description: The VLAN of the trusted server being added. It must be between 1
      and 4094.
    type: int
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for switch createNetworkSwitchDhcpServerPolicyArpInspectionTrustedServer
    description: Complete reference of the createNetworkSwitchDhcpServerPolicyArpInspectionTrustedServer
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-network-switch-dhcp-server-policy-arp-inspection-trusted-server
  - name: Cisco Meraki documentation for switch deleteNetworkSwitchDhcpServerPolicyArpInspectionTrustedServer
    description: Complete reference of the deleteNetworkSwitchDhcpServerPolicyArpInspectionTrustedServer
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-network-switch-dhcp-server-policy-arp-inspection-trusted-server
  - name: Cisco Meraki documentation for switch updateNetworkSwitchDhcpServerPolicyArpInspectionTrustedServer
    description: Complete reference of the updateNetworkSwitchDhcpServerPolicyArpInspectionTrustedServer
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-switch-dhcp-server-policy-arp-inspection-trusted-server
notes:
  - SDK Method used are
    switch.Switch.create_network_switch_dhcp_server_policy_arp_inspection_trusted_server,
    switch.Switch.delete_network_switch_dhcp_server_policy_arp_inspection_trusted_server,
    switch.Switch.update_network_switch_dhcp_server_policy_arp_inspection_trusted_server,
  - Paths used are
    post /networks/{networkId}/switch/dhcpServerPolicy/arpInspection/trustedServers,
    delete /networks/{networkId}/switch/dhcpServerPolicy/arpInspection/trustedServers/{trustedServerId},
    put /networks/{networkId}/switch/dhcpServerPolicy/arpInspection/trustedServers/{trustedServerId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_switch_dhcp_server_policy_arp_inspection_trusted_servers:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    ipv4:
      address: 1.2.3.4
    mac: 00:11:22:33:44:55
    networkId: string
    vlan: 100
- name: Update by id
  cisco.meraki.networks_switch_dhcp_server_policy_arp_inspection_trusted_servers:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    ipv4:
      address: 1.2.3.4
    mac: 00:11:22:33:44:55
    networkId: string
    trustedServerId: string
    vlan: 100
- name: Delete by id
  cisco.meraki.networks_switch_dhcp_server_policy_arp_inspection_trusted_servers:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    networkId: string
    trustedServerId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "trustedServerId": "string",
      "mac": "string",
      "vlan": 0,
      "ipv4": {
        "address": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for iosxr_vrf_global
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: iosxr_vrf_global
short_description: Manages global VRF configuration.
description:
  - This module manages VRF configurations on Cisco IOS-XR devices.
  - It enables playbooks to handle either individual VRFs or the complete VRF collection.
  - It also permits removing non-explicitly stated VRF definitions from the setup.
version_added: 9.0.0
author: Ruchi Pakhle (@Ruchip16)
notes:
  - Tested against Cisco IOS-XR Version 9.0.0
  - This module works with connection C(network_cli).
  - See L(the IOS_XR Platform Options, https://github.com/ansible-collections/cisco.iosxr/blob/main/platform_guide.rst).
  - The module examples uses callback plugin (stdout_callback = yaml) to generate task output in yaml format.
options:
  config:
    description: A dictionary of options for VRF configurations.
    type: list
    elements: dict
    suboptions:
      name:
        description: Name of the VRF.
        type: str
        required: true
      description:
        description: A description for the VRF.
        type: str
      evpn_route_sync:
        description: EVPN Instance VPN ID used to synchronize the VRF route(s).
        type: int
      fallback_vrf:
        description: Fallback VRF name
        type: str
      mhost:
        description: Multicast host stack options
        type: dict
        suboptions:
          afi:
            description: Address Family Identifier (AFI)
            type: str
            choices: ['ipv4', 'ipv6']
          default_interface:
            description: Default interface for multicast.
            type: str
      rd:
        description: VPN Route Distinguisher (RD).
        type: str
      remote_route_filtering:
        description: Enable/Disable remote route filtering per VRF
        type: dict
        suboptions:
          disable:
            description: Disable remote route filtering per VRF
            type: bool
      vpn:
        description: VPN ID for the VRF
        type: dict
        suboptions:
          id:
            description: VPN ID for the VRF.
            type: str
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS-XR device by
        executing the command B(show running-config vrf).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    choices: [parsed, gathered, deleted, merged, replaced, rendered, overridden, purged]
    default: merged
    description:
      - The state the configuration should be left in
      - The states I(rendered), I(gathered) and I(parsed) does not perform any change
        on the device.
      - The state I(rendered) will transform the configuration in C(config) option to
        platform specific CLI commands which will be returned in the I(rendered) key
        within the result. For state I(rendered) active connection to remote host is
        not required.
      - The state I(gathered) will fetch the running configuration from device and transform
        it into structured data in the format as per the resource module argspec and
        the value is returned in the I(gathered) key within the result.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into JSON format as per the resource module parameters and the
        value is returned in the I(parsed) key within the result. The value of C(running_config)
        option should be the same format as the output of command I(show running-config vrf).
        connection to remote host is not required.
      - The state I(purged) removes all the VRF configurations from the
        target device. Use caution with this state.
      - The state I(deleted) only removes the VRF attributes that this module
        manages and does not negate the VRF completely. Thereby, preserving
        address-family related configurations under VRF context.
      - Refer to examples for more details.
    type: str
"""

EXAMPLES = """

# Using merged
#
# Before state:
# -------------
#
# RP/0/0/CPU0:iosxr-02#show running-config vrf
# Fri Feb  9 07:02:35.789 UTC
# !
# vrf test
#

- name: Merge provided configuration with device configuration
  cisco.iosxr.iosxr_vrf_global:
    config:
      - name: VRF4
        description: VRF4 Description
        evpn_route_sync: 793
        fallback_vrf: "test-vrf"
        remote_route_filtering:
          disable: "true"
        rd: "3:4"
        mhost:
          afi: "ipv4"
          default_interface: "Loopback0"
        vpn:
          id: "2:3"
    state: merged

# Task Output:
# ------------
#
# before: []
#
# commands:
# - vrf VRF4
# - description VRF4 Description
# - evpn-route-sync 793
# - fallback-vrf test-vrf
# - mhost ipv4 default-interface Loopback0
# - rd 3:4
# - remote-route-filtering disable
# - vpn id 2:3
#
# after:
# - name: VRF4
#   description: VRF4 Description
#   evpn_route_sync: 793
#   fallback_vrf: "test-vrf"
#   mhost:
#     afi: "ipv4"
#     default_interface: "Loopback0"
#   rd: "3:4"
#   remote_route_filtering:
#     disable: "true"
#   vpn:
#     id: "2:3"
#
# After state:
# ------------
#
# RP/0/0/CPU0:iosxr-02#show running-config vrf
# Sat Feb 20 03:49:43.618 UTC
# vrf VRF4
#  description "VRF4 Description"
#  mhost ipv4 default-interface Loopback0
#  evpn-route-sync 793
#  vpn id 2:3
#  fallback-vrf "test-vrf"
#  remote-route-filtering disable
#  rd "3:4"

# Using replaced
#
# Before state:
# -------------
#
# RP/0/0/CPU0:iosxr-02#show running-config vrf
# Sat Feb 20 03:49:43.618 UTC
# vrf VRF4
#  description "VRF4 Description"
#  mhost ipv4 default-interface Loopback0
#  evpn-route-sync 793
#  vpn id 2:3
#  fallback-vrf "test-vrf"
#  remote-route-filtering disable
#  rd "3:4"

- name: Replace the provided configuration with the existing running configuration
  cisco.iosxr.iosxr_vrf_global:
    config:
      - name: VRF7
        description: VRF7 description
        evpn_route_sync: 398
        fallback_vrf: "replaced-vrf"
        remote_route_filtering:
          disable: "true"
        rd: "67:9"
        mhost:
          afi: "ipv4"
          default_interface: "Loopback0"
        vpn:
          id: "4:5"
    state: replaced

# Task Output:
# ------------
#
# before:
# - name: VRF4
#   description: VRF4 Description
#   evpn_route_sync: 793
#   fallback_vrf: "test-vrf"
#   mhost:
#     afi: "ipv4"
#     default_interface: "Loopback0"
#   rd: "3:4"
#   remote_route_filtering:
#     disable: "true"
#   vpn:
#     id: "2:3"
#
# commands:
# - vrf VRF4
# - no vpn id 2:3
# - vrf VRF7
# - description VRF7 description
# - evpn-route-sync 398
# - fallback-vrf replaced-vrf
# - mhost ipv4 default-interface Loopback0
# - rd 6:9
# - remote-route-filtering disable
# - vpn id 4:5
#
# after:
#   - name: VRF4
#     description: VRF4 Description
#     evpn_route_sync: 793
#     fallback_vrf: "test-vrf"
#     mhost:
#       afi: "ipv4"
#       default_interface: "Loopback0"
#     rd: "3:4"
#     remote_route_filtering:
#       disable: "true"
#   - name: VRF7
#     description: VRF7 description
#     evpn_route_sync: 398
#     fallback_vrf: "replaced-vrf"
#     remote_route_filtering:
#       disable: true
#     rd: "67:9"
#     mhost:
#       afi: "ipv4"
#       default_interface: "Loopback0"
#     vpn:
#       id: "4:5"
#
# After state:
# ------------
#
# RP/0/RP0/CPU0:ios(config)#show running-config vrf
# Sun Mar 10 16:48:53.204 UTC
# vrf VRF4
#  mhost ipv4 default-interface Loopback0
#  evpn-route-sync 793
#  description VRF4 Description
#  fallback-vrf test-vrf
#  remote-route-filtering disable
#  rd 3:4
# !
# vrf VRF7
#  mhost ipv4 default-interface Loopback0
#  evpn-route-sync 398
#  description VRF7 description
#  vpn id 4:5
#  fallback-vrf replaced-vrf
#  remote-route-filtering disable
#  rd 67:9
#  !
# !

# Using overridden
#
# Before state:
# -------------
#
# RP/0/RP0/CPU0:ios(config)#show running-config vrf
# Sun Mar 10 16:48:53.204 UTC
# vrf VRF4
#  mhost ipv4 default-interface Loopback0
#  evpn-route-sync 793
#  description VRF4 Description
#  fallback-vrf test-vrf
#  remote-route-filtering disable
#  rd 3:4
#  !
# !
# vrf VRF7
#  mhost ipv4 default-interface Loopback0
#  evpn-route-sync 398
#  description VRF7 description
#  vpn id 4:5
#  fallback-vrf replaced-vrf
#  remote-route-filtering disable
#  rd 67:9
#  !
# !

- name: Override the provided configuration with the existing running configuration
  cisco.iosxr.iosxr_vrf_global:
    state: overridden
    config:
      - name: VRF6
        description: VRF6 Description
        evpn_route_sync: 101
        fallback_vrf: "overridden-vrf"
        remote_route_filtering:
          disable: "true"
        rd: "9:8"
        mhost:
          afi: "ipv4"
          default_interface: "Loopback0"
        vpn:
          id: "23:3"

# Task Output:
# ------------
#
# before:
# - name: VRF4
#   description: VRF4 Description
#   evpn_route_sync: 793
#   fallback_vrf: "test-vrf"
#   mhost:
#     afi: "ipv4"
#     default_interface: "Loopback0"
#   rd: "3:4"
#   remote_route_filtering:
#     disable: "true"
# - name: VRF7
#   description: VRF7 description
#   evpn_route_sync: 398
#   fallback_vrf: "replaced-vrf"
#   remote_route_filtering:
#     disable: true
#   rd: "67:9"
#   mhost:
#     afi: "ipv4"
#     default_interface: "Loopback0"
#   vpn:
#     id: "4:5"
#
# commands:
# - vrf VRF4
# - no description VRF4 Description
# - no evpn-route-sync 793
# - no fallback-vrf test-vrf
# - no mhost ipv4 default-interface Loopback0
# - no rd 3:4
# - no remote-route-filtering disable
# - vrf VRF7
# - no description VRF7 description
# - no evpn-route-sync 398
# - no fallback-vrf replaced-vrf
# - no mhost ipv4 default-interface Loopback0
# - no rd 67:9
# - no remote-route-filtering disable
# - no vpn id 4:5
# - vrf VRF6
# - description VRF6 Description
# - evpn-route-sync 101
# - fallback-vrf overridden-vrf
# - mhost ipv4 default-interface Loopback0
# - rd 9:8
# - remote-route-filtering disable
# - vpn id 23:3
#
# after:
# - name: VRF4
# - name: VRF6
#   description: VRF6 Description
#   evpn_route_sync: 101
#   fallback_vrf: "overridden-vrf"
#   remote_route_filtering:
#     disable: "true"
#   rd: "9:8"
#   mhost:
#     afi: "ipv4"
#     default_interface: "Loopback0"
#   vpn:
#     id: "23:3"
# - name: VRF7
#
# After state:
# -------------
# RP/0/RP0/CPU0:ios(config)#show running-config vrf
# Sun Mar 10 16:54:53.007 UTC
# vrf VRF4
# vrf VRF6
#  mhost ipv4 default-interface Loopback0
#  evpn-route-sync 101
#  description VRF6 Description
#  vpn id 23:3
#  fallback-vrf overridden-vrf
#  remote-route-filtering disable
#  rd 9:8
# vrf VRF7

# Using deleted
#
# Before state:
# -------------
#
# RP/0/RP0/CPU0:ios(config)#show running-config vrf
# Sun Mar 10 16:54:53.007 UTC
# vrf VRF4
# vrf VRF6
#  mhost ipv4 default-interface Loopback0
#  evpn-route-sync 101
#  description VRF6 Description
#  vpn id 23:3
#  fallback-vrf overridden-vrf
#  remote-route-filtering disable
#  rd 9:8
# vrf VRF7

- name: Delete the provided configuration
  cisco.iosxr.iosxr_vrf_global:
    config:
    state: deleted

# Task Output:
# ------------
#
# before:
# - name: VRF4
# - name: VRF6
#   description: VRF6 Description
#   evpn_route_sync: 101
#   fallback_vrf: "overridden-vrf"
#   remote_route_filtering:
#     disable: "true"
#   rd: "9:8"
#   mhost:
#     afi: "ipv4"
#     default_interface: "Loopback0"
#   vpn:
#     id: "23:3"
# - name: VRF7

# commands:
# - vrf VRF4
# - vrf VRF6
# - no description VRF6 Description
# - no evpn-route-sync 101
# - no fallback-vrf overridden-vrf
# - no mhost ipv4 default-interface Loopback0
# - no rd 9:8
# - no remote-route-filtering disable
# - no vpn id 23:3
# - vrf VRF7
#
# after:
# - name: VRF4
# - name: VRF6
# - name: VRF7
#
# After state:
# ------------
#
# RP/0/RP0/CPU0:ios(config)#show running-config vrf
# Sun Mar 10 17:02:38.981 UTC
# vrf VRF4
# vrf VRF6
# vrf VRF7

# Using purged
#
# Before state:
# -------------
#
# RP/0/RP0/CPU0:ios(config)#show running-config vrf
# vrf VRF4
# vrf VRF6
# vrf VRF7

- name: Purge all the configuration from the device
  cisco.iosxr.iosxr_vrf_global:
    state: purged

# Task Output:
# ------------
#
# before:
# - name: VRF4
# - name: VRF6
# - name: VRF7
#
# commands:
# - no vrf VRF4
# - no vrf VRF6
# - no vrf VRF7
#
# after: []
#
# After state:
# -------------
# RP/0/RP0/CPU0:ios(config)#show running-config vrf
# Sun Mar 10 17:02:38.981 UTC
# -

# Using rendered
#
- name: Render provided configuration with device configuration
  cisco.iosxr.iosxr_vrf_global:
    config:
      - name: VRF4
        description: VRF4 Description
        evpn_route_sync: 793
        fallback_vrf: "test-vrf"
        remote_route_filtering:
          disable: "true"
        rd: "3:4"
        mhost:
          afi: "ipv4"
          default_interface: "Loopback0"
        vpn:
          id: "2:3"
    state: rendered

# Task Output:
# ------------
#
# rendered:
# - vrf VRF4
# - description VRF4 Description
# - evpn-route-sync 793
# - fallback-vrf test-vrf
# - mhost ipv4 default-interface Loopback0
# - rd 3:4
# - remote-route-filtering disable
# - vpn id 2:3

# Using gathered
#
# Before state:
# -------------
#
# RP/0/RP0/CPU0:ios(config)#show running-config vrf
# Sun Mar 10 17:02:38.981 UTC
# vrf VRF4
#  description "VRF4 Description"
#  mhost ipv4 default-interface Loopback0
#  evpn-route-sync 793
#  vpn id 2:3
#  fallback-vrf "test-vrf"
#  remote-route-filtering disable
#  rd "3:4"

- name: Gather existing running configuration
  cisco.iosxr.iosxr_vrf_global:
    state: gathered

# Task Output:
# ------------
#
# gathered:
# - name: VRF4
#   description: VRF4 Description
#   evpn_route_sync: 793
#   fallback_vrf: "test-vrf"
#   mhost:
#     afi: "ipv4"
#     default_interface: "Loopback0"
#   rd: "3:4"
#   remote_route_filtering:
#     disable: "true"
#   vpn:
#     id: "2:3"

# Using parsed
#
# File: parsed.cfg
# ----------------
#
# vrf test
#  description "This is test VRF"
#  mhost ipv4 default-interface Loopback0
#  evpn-route-sync 456
#  vpn id 56
#  fallback-vrf "test-vrf"
#  remote-route-filtering disable
#  rd "testing"
#  !
# !
# vrf my_vrf
#  mhost ipv4 default-interface Loopback0
#  evpn-route-sync 235
#  description "this is sample vrf for feature testing"
#  fallback-vrf "parsed-vrf"
#  rd "2:3"
#  remote-route-filtering disable
#  vpn id 23
#  !
# !

- name: Parse the provided configuration
  cisco.iosxr.iosxr_vrf_global:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Task Output:
# ------------
#
# parsed:
# - description: This is test VRF
#   evpn_route_sync: 456
#   fallback_vrf: test-vrf
#   mhost:
#     afi: ipv4
#     default_interface: Loopback0
#   name: test
#   rd: testing
#   remote_route_filtering:
#     disable: true
#   vpn:
#     id: '56'
# - description: this is sample vrf for feature testing
#   evpn_route_sync: 235
#   fallback_vrf: parsed-vrf
#   mhost:
#     afi: ipv4
#     default_interface: Loopback0
#   name: my_vrf
#   rd: '2:3'
#   remote_route_filtering:
#     disable: true
#   vpn:
#     id: '23'
"""

RETURN = """
before:
  description: The configuration prior to the model invocation.
  returned: always
  type: dict
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The resulting configuration model invocation.
  returned: when changed
  type: dict
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample:
  - vrf VRF7
  - description VRF7 description
  - rd: 67:9
  - fallback-vrf replaced-vrf
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
  - vrf VRF4
  - description VRF4 Description
  - evpn-route-sync 793
  - fallback-vrf parsed-vrf
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.vrf_global.vrf_global import (
    Vrf_globalArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.config.vrf_global.vrf_global import (
    Vrf_global,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Vrf_globalArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Vrf_global(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Filter plugin file for usable_range
"""

from __future__ import absolute_import, division, print_function

from ipaddress import IPv4Network, IPv6Network

from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddress_utils import (
    _need_ipaddress,
    ip_network,
)
from ansible_collections.ansible.utils.plugins.plugin_utils.base.utils import _validate_args


__metaclass__ = type

from ansible.errors import AnsibleFilterError
from ansible.module_utils.common.text.converters import to_text


DOCUMENTATION = """
    name: usable_range
    author: Priyam Sahoo (@priyamsahoo)
    version_added: "2.3.0"
    short_description: Expand the usable IP addresses
    description:
        - For a given IP address (IPv4 or IPv6) in CIDR form, the plugin generates a list of usable IP addresses belonging to the network.
    options:
        ip:
            description:
            - A string that represents an IP address of network in CIDR form
            - 'For example: C(10.0.0.0/24) or C(2001:db8:abcd:0012::0/124)'
            type: str
            required: True
    notes:
"""

EXAMPLES = r"""

#### Simple examples

- name: Expand and produce list of usable IP addresses in 10.0.0.0/28
  ansible.builtin.set_fact:
    data: "{{ '10.0.0.0/28' | ansible.utils.usable_range }}"

# TASK [Expand and produce list of usable IP addresses in 10.0.0.0/28] ************************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": {
#             "number_of_ips": 16,
#             "usable_ips": [
#                 "10.0.0.0",
#                 "10.0.0.1",
#                 "10.0.0.2",
#                 "10.0.0.3",
#                 "10.0.0.4",
#                 "10.0.0.5",
#                 "10.0.0.6",
#                 "10.0.0.7",
#                 "10.0.0.8",
#                 "10.0.0.9",
#                 "10.0.0.10",
#                 "10.0.0.11",
#                 "10.0.0.12",
#                 "10.0.0.13",
#                 "10.0.0.14",
#                 "10.0.0.15"
#             ]
#         }
#     },
#     "changed": false
# }

- name: Expand and produce list of usable IP addresses in 2001:db8:abcd:0012::0/126
  ansible.builtin.set_fact:
    data1: "{{ '2001:db8:abcd:0012::0/126' | ansible.utils.usable_range }}"

# TASK [Expand and produce list of usable IP addresses in 2001:db8:abcd:0012::0/126] ***
# ok: [localhost] => {
#     "ansible_facts": {
#         "data1": {
#             "number_of_ips": 4,
#             "usable_ips": [
#                 "2001:db8:abcd:12::",
#                 "2001:db8:abcd:12::1",
#                 "2001:db8:abcd:12::2",
#                 "2001:db8:abcd:12::3"
#             ]
#         }
#     },
#     "changed": false
# }

- name: Expand and produce list of usable IP addresses in 10.1.1.1
  ansible.builtin.set_fact:
    data: "{{ '10.1.1.1' | ansible.utils.usable_range }}"

# TASK [Expand and produce list of usable IP addresses in 10.1.1.1] ***************************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": {
#             "number_of_ips": 1,
#             "usable_ips": [
#                 "10.1.1.1"
#             ]
#         }
#     },
#     "changed": false
# }

#### Simple Use-case (looping through the list result)

- name: Expand and produce list of usable IP addresses in 127.0.0.0/28
  ansible.builtin.set_fact:
    data1: "{{ '127.0.0.0/28' | ansible.utils.usable_range }}"

- name: Ping all but first IP addresses from the generated list
  shell: "ping -c 1 {{ item }}"
  loop: "{{ data1.usable_ips[1:] }}"

# TASK [Expand and produce list of usable IP addresses in 127.0.0.0/28] ******************************
# ok: [localhost]

# TASK [Ping all but first IP addresses from the generated list] *************************************
# changed: [localhost] => (item=127.0.0.1)
# changed: [localhost] => (item=127.0.0.2)
# changed: [localhost] => (item=127.0.0.3)
# changed: [localhost] => (item=127.0.0.4)
# changed: [localhost] => (item=127.0.0.5)
# changed: [localhost] => (item=127.0.0.6)
# changed: [localhost] => (item=127.0.0.7)
# changed: [localhost] => (item=127.0.0.8)
# changed: [localhost] => (item=127.0.0.9)
# changed: [localhost] => (item=127.0.0.10)
# changed: [localhost] => (item=127.0.0.11)
# changed: [localhost] => (item=127.0.0.12)
# changed: [localhost] => (item=127.0.0.13)
# changed: [localhost] => (item=127.0.0.14)
# changed: [localhost] => (item=127.0.0.15)
"""

RETURN = """
    data:
        description:
        - Total number of usable IP addresses under the key C(number_of_ips)
        - List of usable IP addresses under the key C(usable_ips)
"""


@_need_ipaddress
def _usable_range(ip):
    """Expand the usable IP addresses"""

    params = {"ip": ip}
    _validate_args("usable_range", DOCUMENTATION, params)

    try:
        if ip_network(ip).version == 4:
            ips = [to_text(usable_ips) for usable_ips in IPv4Network(to_text(ip))]
            no_of_ips = IPv4Network(to_text(ip)).num_addresses
        if ip_network(ip).version == 6:
            ips = [to_text(usable_ips) for usable_ips in IPv6Network(to_text(ip))]
            no_of_ips = IPv6Network(to_text(ip)).num_addresses

    except Exception as e:
        raise AnsibleFilterError(
            "Error while using plugin 'usable_range': {msg}".format(msg=to_text(e)),
        )

    return {"usable_ips": ips, "number_of_ips": no_of_ips}


class FilterModule(object):
    """usable_range"""

    def filters(self):
        """a mapping of filter names to functions"""
        return {"usable_range": _usable_range}



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_wireless_ssids_firewall_isolation_allowlist_entries_info
short_description: Information module for organizations _wireless _ssids _firewall
  _isolation _allowlist _entries
description:
  - Get all organizations _wireless _ssids _firewall _isolation _allowlist _entries.
  - List the L2 isolation allow list MAC entry in an organization.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 1000.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  networkIds:
    description:
      - NetworkIds query parameter. NetworkIds array to filter out results.
    elements: str
    type: list
  ssids:
    description:
      - Ssids query parameter. Ssids number array to filter out results.
    elements: int
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless getOrganizationWirelessSsidsFirewallIsolationAllowlistEntries
    description: Complete reference of the getOrganizationWirelessSsidsFirewallIsolationAllowlistEntries
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-wireless-ssids-firewall-isolation-allowlist-entries
notes:
  - SDK Method used are
    wireless.Wireless.get_organization_wireless_ssids_firewall_isolation_allowlist_entries,
  - Paths used are
    get /organizations/{organizationId}/wireless/ssids/firewall/isolation/allowlist/entries,
"""

EXAMPLES = r"""
- name: Get all organizations _wireless _ssids _firewall _isolation _allowlist _entries
  cisco.meraki.organizations_wireless_ssids_firewall_isolation_allowlist_entries_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    networkIds: []
    ssids: []
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "entryId": "string",
          "createdAt": "string",
          "lastUpdatedAt": "string",
          "description": "string",
          "client": {
            "mac": "string"
          },
          "ssid": {
            "id": "string",
            "name": "string",
            "number": 0
          },
          "network": {
            "id": "string",
            "name": "string"
          }
        }
      ],
      "meta": {
        "counts": {
          "items": {
            "total": 0,
            "remaining": 0
          }
        }
      }
    }
"""



#!/usr/bin/python
# Copyright (c) 2018, David Kainz <dkainz@mgit.at> <dave.jokain@gmx.at>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: openssh_cert
author: "David Kainz (@lolcube)"
short_description: Generate OpenSSH host or user certificates
description:
  - Generate and regenerate OpenSSH host or user certificates.
requirements:
  - "ssh-keygen"
extends_documentation_fragment:
  - ansible.builtin.files
  - community.crypto._attributes
  - community.crypto._attributes.files
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
  safe_file_operations:
    support: full
  idempotent:
    support: partial
    details:
      - The module is not idempotent if O(force=true) or O(regenerate=always).
      - If relative timestamps are used and O(ignore_timestamps=false) (default), the module is not idempotent.
options:
  state:
    description:
      - Whether the host or user certificate should exist or not, taking action if the state is different from what is stated.
    type: str
    default: "present"
    choices: ['present', 'absent']
  type:
    description:
      - Whether the module should generate a host or a user certificate.
      - Required if O(state) is V(present).
    type: str
    choices: ['host', 'user']
  force:
    description:
      - Should the certificate be regenerated even if it already exists and is valid.
      - Equivalent to O(regenerate=always).
    type: bool
    default: false
  path:
    description:
      - Path of the file containing the certificate.
    type: path
    required: true
  regenerate:
    description:
      - When V(never) the task will fail if a certificate already exists at O(path) and is unreadable otherwise a new certificate
        will only be generated if there is no existing certificate.
      - When V(fail) the task will fail if a certificate already exists at O(path) and does not match the module's options.
      - When V(partial_idempotence) an existing certificate will be regenerated based on O(serial_number), O(signature_algorithm),
        O(type), O(valid_from), O(valid_to), O(valid_at), and O(principals). O(valid_from) and O(valid_to) can be excluded
        by O(ignore_timestamps=true).
      - When V(full_idempotence) O(identifier), O(options), O(public_key), and O(signing_key) are also considered when compared
        against an existing certificate.
      - V(always) is equivalent to O(force=true).
    type: str
    choices:
      - never
      - fail
      - partial_idempotence
      - full_idempotence
      - always
    default: partial_idempotence
    version_added: 1.8.0
  signature_algorithm:
    description:
      - As of OpenSSH 8.2 the SHA-1 signature algorithm for RSA keys has been disabled and C(ssh) will refuse host certificates
        signed with the SHA-1 algorithm. OpenSSH 8.1 made V(rsa-sha2-512) the default algorithm when acting as a CA and signing
        certificates with a RSA key. However, for OpenSSH versions less than 8.1 the SHA-2 signature algorithms, V(rsa-sha2-256)
        or V(rsa-sha2-512), must be specified using this option if compatibility with newer C(ssh) clients is required. Conversely
        if hosts using OpenSSH version 8.2 or greater must remain compatible with C(ssh) clients using OpenSSH less than 7.2,
        then V(ssh-rsa) can be used when generating host certificates (a corresponding change to the sshd_config to add V(ssh-rsa)
        to the C(CASignatureAlgorithms) keyword is also required).
      - Using any value for this option with a non-RSA O(signing_key) will cause this module to fail.
      - 'Note: OpenSSH versions prior to 7.2 do not support SHA-2 signature algorithms for RSA keys and OpenSSH versions prior
        to 7.3 do not support SHA-2 signature algorithms for certificates.'
      - See U(https://www.openssh.com/txt/release-8.2) for more information.
    type: str
    choices:
      - ssh-rsa
      - rsa-sha2-256
      - rsa-sha2-512
    version_added: 1.10.0
  signing_key:
    description:
      - The path to the private openssh key that is used for signing the public key in order to generate the certificate.
      - If the private key is on a PKCS#11 token (O(pkcs11_provider)), set this to the path to the public key instead.
      - Required if O(state) is V(present).
    type: path
  pkcs11_provider:
    description:
      - To use a signing key that resides on a PKCS#11 token, set this to the name (or full path) of the shared library to
        use with the token. Usually C(libpkcs11.so).
      - If this is set, O(signing_key) needs to point to a file containing the public key of the CA.
    type: str
    version_added: 1.1.0
  use_agent:
    description:
      - Should the ssh-keygen use a CA key residing in a ssh-agent.
    type: bool
    default: false
    version_added: 1.3.0
  public_key:
    description:
      - The path to the public key that will be signed with the signing key in order to generate the certificate.
      - Required if O(state) is V(present).
    type: path
  valid_from:
    description:
      - 'The point in time the certificate is valid from. Time can be specified either as relative time or as absolute timestamp.
        Time will always be interpreted as UTC. Valid formats are: C([+-]timespec | YYYY-MM-DD | YYYY-MM-DDTHH:MM:SS | YYYY-MM-DD
        HH:MM:SS | always) where timespec can be an integer + C([w | d | h | m | s]) (for example V(+32w1d2h)). Note that
        if using relative time this module is NOT idempotent.'
      - The value V(always) is only supported for OpenSSH 7.7 and greater, however, the value V(1970-01-01T00:00:01) can be
        used with earlier versions as an equivalent expression.
      - To ignore this value during comparison with an existing certificate set O(ignore_timestamps=true).
      - Required if O(state) is V(present).
    type: str
  valid_to:
    description:
      - 'The point in time the certificate is valid to. Time can be specified either as relative time or as absolute timestamp.
        Time will always be interpreted as UTC. Valid formats are: C([+-]timespec | YYYY-MM-DD | YYYY-MM-DDTHH:MM:SS | YYYY-MM-DD
        HH:MM:SS | forever) where timespec can be an integer + C([w | d | h | m | s]) (for example V(+32w1d2h)). Note that
        if using relative time this module is NOT idempotent.'
      - To ignore this value during comparison with an existing certificate set O(ignore_timestamps=true).
      - Required if O(state) is V(present).
    type: str
  valid_at:
    description:
      - Check if the certificate is valid at a certain point in time. If it is not the certificate will be regenerated. Time
        will always be interpreted as UTC. Mainly to be used with relative timespec for O(valid_from) and / or O(valid_to).
        Note that if using relative time this module is NOT idempotent.
    type: str
  ignore_timestamps:
    description:
      - Whether the O(valid_from) and O(valid_to) timestamps should be ignored for idempotency checks.
      - However, the values will still be applied to a new certificate if it meets any other necessary conditions for generation/regeneration.
    type: bool
    default: false
    version_added: 2.2.0
  principals:
    description:
      - Certificates may be limited to be valid for a set of principal (user/host) names. By default, generated certificates
        are valid for all users or hosts.
    type: list
    elements: str
  options:
    description:
      - 'Specify certificate options when signing a key. The option that are valid for user certificates are:'
      - 'V(clear): Clear all enabled permissions. This is useful for clearing the default set of permissions so permissions
        may be added individually.'
      - 'V(force-command=command): Forces the execution of command instead of any shell or command specified by the user when
        the certificate is used for authentication.'
      - 'V(no-agent-forwarding): Disable ssh-agent forwarding (permitted by default).'
      - 'V(no-port-forwarding): Disable port forwarding (permitted by default).'
      - 'V(no-pty): Disable PTY allocation (permitted by default).'
      - 'V(no-user-rc): Disable execution of C(~/.ssh/rc) by sshd (permitted by default).'
      - 'V(no-x11-forwarding): Disable X11 forwarding (permitted by default).'
      - 'V(permit-agent-forwarding): Allows ssh-agent forwarding.'
      - 'V(permit-port-forwarding): Allows port forwarding.'
      - 'V(permit-pty): Allows PTY allocation.'
      - 'V(permit-user-rc): Allows execution of C(~/.ssh/rc) by sshd.'
      - 'V(permit-x11-forwarding): Allows X11 forwarding.'
      - 'V(source-address=address_list): Restrict the source addresses from which the certificate is considered valid. The
        C(address_list) is a comma-separated list of one or more address/netmask pairs in CIDR format.'
      - At present, no options are valid for host keys.
    type: list
    elements: str
  identifier:
    description:
      - Specify the key identity when signing a public key. The identifier that is logged by the server when the certificate
        is used for authentication.
    type: str
  serial_number:
    description:
      - 'Specify the certificate serial number. The serial number is logged by the server when the certificate is used for
        authentication. The certificate serial number may be used in a KeyRevocationList. The serial number may be omitted
        for checks, but must be specified again for a new certificate. Note: The default value set by ssh-keygen is 0.'
      - This option accepts an B(integer). If you want to provide serial numbers as colon-separated hex strings, such as C(11:22:33),
        you need to convert them to an integer with P(community.crypto.parse_serial#filter).
    type: int

seealso:
  - plugin: community.crypto.parse_serial
    plugin_type: filter
"""

EXAMPLES = r"""
---
- name: Generate an OpenSSH user certificate that is valid forever and for all users
  community.crypto.openssh_cert:
    type: user
    signing_key: /path/to/private_key
    public_key: /path/to/public_key.pub
    path: /path/to/certificate
    valid_from: always
    valid_to: forever

# Generate an OpenSSH host certificate that is valid for 32 weeks from now and will be regenerated
# if it is valid for less than 2 weeks from the time the module is being run
- name: Generate an OpenSSH host certificate with valid_from, valid_to and valid_at parameters
  community.crypto.openssh_cert:
    type: host
    signing_key: /path/to/private_key
    public_key: /path/to/public_key.pub
    path: /path/to/certificate
    valid_from: +0s
    valid_to: +32w
    valid_at: +2w
    ignore_timestamps: true

- name: Generate an OpenSSH host certificate that is valid forever and only for example.com and examplehost
  community.crypto.openssh_cert:
    type: host
    signing_key: /path/to/private_key
    public_key: /path/to/public_key.pub
    path: /path/to/certificate
    valid_from: always
    valid_to: forever
    principals:
      - example.com
      - examplehost

- name: Generate an OpenSSH host Certificate that is valid from 21.1.2001 to 21.1.2019
  community.crypto.openssh_cert:
    type: host
    signing_key: /path/to/private_key
    public_key: /path/to/public_key.pub
    path: /path/to/certificate
    valid_from: "2001-01-21"
    valid_to: "2019-01-21"

- name: Generate an OpenSSH user Certificate with clear and force-command option
  community.crypto.openssh_cert:
    type: user
    signing_key: /path/to/private_key
    public_key: /path/to/public_key.pub
    path: /path/to/certificate
    valid_from: always
    valid_to: forever
    options:
      - "clear"
      - "force-command=/tmp/bla/foo"

- name: Generate an OpenSSH user certificate using a PKCS#11 token
  community.crypto.openssh_cert:
    type: user
    signing_key: /path/to/ca_public_key.pub
    pkcs11_provider: libpkcs11.so
    public_key: /path/to/public_key.pub
    path: /path/to/certificate
    valid_from: always
    valid_to: forever
"""

RETURN = r"""
type:
  description: Type of the certificate (host or user).
  returned: changed or success
  type: str
  sample: host
filename:
  description: Path to the certificate.
  returned: changed or success
  type: str
  sample: /tmp/certificate-cert.pub
info:
  description: Information about the certificate. Output of C(ssh-keygen -L -f).
  returned: change or success
  type: list
  elements: str
"""

import os
import typing as t

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.crypto.plugins.module_utils._openssh.backends.common import (
    KeygenCommand,
    OpensshModule,
    PrivateKey,
)
from ansible_collections.community.crypto.plugins.module_utils._openssh.certificate import (
    OpensshCertificate,
    OpensshCertificateTimeParameters,
    parse_option_list,
)
from ansible_collections.community.crypto.plugins.module_utils._version import (
    LooseVersion,
)


class Certificate(OpensshModule):
    def __init__(self, module: AnsibleModule) -> None:
        super().__init__(module=module)
        self.ssh_keygen = KeygenCommand(self.module)

        self.identifier: str = self.module.params["identifier"] or ""
        self.options: list[str] = self.module.params["options"] or []
        self.path: str = self.module.params["path"]
        self.pkcs11_provider: str | None = self.module.params["pkcs11_provider"]
        self.principals: list[str] = self.module.params["principals"] or []
        self.public_key: str | None = self.module.params["public_key"]
        self.regenerate: t.Literal[
            "never",
            "fail",
            "partial_idempotence",
            "full_idempotence",
            "always",
        ] = (
            self.module.params["regenerate"]
            if not self.module.params["force"]
            else "always"
        )
        self.serial_number: int | None = self.module.params["serial_number"]
        self.signature_algorithm: (
            t.Literal["ssh-rsa", "rsa-sha2-256", "rsa-sha2-512"] | None
        ) = self.module.params["signature_algorithm"]
        self.signing_key: str | None = self.module.params["signing_key"]
        self.state: t.Literal["absent", "present"] = self.module.params["state"]
        self.type: t.Literal["host", "user"] | None = self.module.params["type"]
        self.use_agent: bool = self.module.params["use_agent"]
        self.valid_at: str | None = self.module.params["valid_at"]
        self.ignore_timestamps: bool = self.module.params["ignore_timestamps"]

        self._check_if_base_dir(self.path)

        if self.state == "present":
            self._validate_parameters()

        self.data: OpensshCertificate | None = None
        self.original_data: OpensshCertificate | None = None
        if self._exists():
            self._load_certificate()

        self.time_parameters: OpensshCertificateTimeParameters | None = None
        if self.state == "present":
            self._set_time_parameters()

    def _validate_parameters(self) -> None:
        for path in (self.public_key, self.signing_key):
            if (
                path is not None
            ):  # should never be None, but the type checker doesn't know
                self._check_if_base_dir(path)

        if self.options and self.type == "host":
            self.module.fail_json(
                msg="Options can only be used with user certificates."
            )

        if self.use_agent:
            self._use_agent_available()

    def _use_agent_available(self) -> None:
        ssh_version = self._get_ssh_version()
        if not ssh_version:
            self.module.fail_json(msg="Failed to determine ssh version")
        elif LooseVersion(ssh_version) < LooseVersion("7.6"):
            self.module.fail_json(
                msg="Signing with CA key in ssh agent requires ssh 7.6 or newer."
                + f" Your version is: {ssh_version}"
            )

    def _exists(self) -> bool:
        return os.path.exists(self.path)

    def _load_certificate(self) -> None:
        try:
            self.original_data = OpensshCertificate.load(self.path)
        except (TypeError, ValueError) as e:
            if self.regenerate in ("never", "fail"):
                self.module.fail_json(msg=f"Unable to read existing certificate: {e}")
            self.module.warn(f"Unable to read existing certificate: {e}")

    def _set_time_parameters(self) -> None:
        try:
            self.time_parameters = OpensshCertificateTimeParameters(
                valid_from=self.module.params["valid_from"],
                valid_to=self.module.params["valid_to"],
            )
        except ValueError as e:
            self.module.fail_json(msg=str(e))

    def _execute(self) -> None:
        if self.state == "present":
            if self._should_generate():
                self._generate()
                self._update_permissions(self.path)
        else:
            if self._exists():
                self._remove()

    def _should_generate(self) -> bool:
        if self.regenerate == "never":
            return self.original_data is None
        if self.regenerate == "fail":
            if self.original_data and not self._is_fully_valid():
                self.module.fail_json(
                    msg="Certificate does not match the provided options.",
                    cert=get_cert_dict(self.original_data),
                )
            return self.original_data is None
        if self.regenerate == "partial_idempotence":
            return self.original_data is None or not self._is_partially_valid()
        if self.regenerate == "full_idempotence":
            return self.original_data is None or not self._is_fully_valid()
        return True

    def _is_fully_valid(self) -> bool:
        if self.original_data is None:
            raise AssertionError(
                "Contract violation original_data not provided"
            )  # pragma: no cover
        if self.public_key is None:
            raise AssertionError(
                "Contract violation public_key not provided"
            )  # pragma: no cover
        if self.signing_key is None:
            raise AssertionError(
                "Contract violation signing_key not provided"
            )  # pragma: no cover
        return self._is_partially_valid() and all(
            [
                self._compare_options() if self.original_data.type == "user" else True,
                self.original_data.key_id == self.identifier,
                self.original_data.public_key
                == self._get_key_fingerprint(self.public_key),
                self.original_data.signing_key
                == self._get_key_fingerprint(self.signing_key),
            ]
        )

    def _is_partially_valid(self) -> bool:
        if self.original_data is None:
            raise AssertionError(
                "Contract violation original_data not provided"
            )  # pragma: no cover
        return all(
            [
                set(self.original_data.principals) == set(self.principals),
                (
                    self.original_data.signature_type == self.signature_algorithm
                    if self.signature_algorithm
                    else True
                ),
                (
                    self.original_data.serial == self.serial_number
                    if self.serial_number is not None
                    else True
                ),
                self.original_data.type == self.type,
                self._compare_time_parameters(),
            ]
        )

    def _compare_time_parameters(self) -> bool:
        if self.original_data is None:
            raise AssertionError(
                "Contract violation original_data not provided"
            )  # pragma: no cover
        try:
            original_time_parameters = OpensshCertificateTimeParameters(
                valid_from=self.original_data.valid_after,
                valid_to=self.original_data.valid_before,
            )
        except ValueError as e:
            return self.module.fail_json(msg=str(e))

        if self.ignore_timestamps:
            return original_time_parameters.within_range(self.valid_at)

        return all(
            [
                original_time_parameters == self.time_parameters,
                original_time_parameters.within_range(self.valid_at),
            ]
        )

    def _compare_options(self) -> bool:
        if self.original_data is None:
            raise AssertionError(
                "Contract violation original_data not provided"
            )  # pragma: no cover
        try:
            critical_options, extensions = parse_option_list(self.options)
        except ValueError as e:
            return self.module.fail_json(msg=str(e))

        return all(
            [
                set(self.original_data.critical_options) == set(critical_options),
                set(self.original_data.extensions) == set(extensions),
            ]
        )

    def _get_key_fingerprint(self, path: str) -> str:
        private_key_content = self.ssh_keygen.get_private_key(
            private_key_path=path, check_rc=True
        )[1]
        return PrivateKey.from_string(private_key_content).fingerprint

    @OpensshModule.trigger_change
    @OpensshModule.skip_if_check_mode
    def _generate(self) -> None:
        try:
            temp_certificate = self._generate_temp_certificate()
            self._safe_secure_move([(temp_certificate, self.path)])
        except OSError as e:
            self.module.fail_json(
                msg=f"Unable to write certificate to {self.path}: {e}"
            )

        try:
            self.data = OpensshCertificate.load(self.path)
        except (TypeError, ValueError) as e:
            self.module.fail_json(msg=f"Unable to read new certificate: {e}")

    def _generate_temp_certificate(self) -> str:
        if self.public_key is None:
            raise AssertionError(
                "Contract violation public_key not provided"
            )  # pragma: no cover
        if self.signing_key is None:
            raise AssertionError(
                "Contract violation signing_key not provided"
            )  # pragma: no cover
        if self.time_parameters is None:
            raise AssertionError(
                "Contract violation time_parameters not provided"
            )  # pragma: no cover

        key_copy = os.path.join(self.module.tmpdir, os.path.basename(self.public_key))

        try:
            self.module.preserved_copy(self.public_key, key_copy)
        except OSError as e:
            self.module.fail_json(msg=f"Unable to stage temporary key: {e}")
        self.module.add_cleanup_file(key_copy)

        self.ssh_keygen.generate_certificate(
            certificate_path=key_copy,
            identifier=self.identifier,
            options=self.options,
            pkcs11_provider=self.pkcs11_provider,
            principals=self.principals,
            serial_number=self.serial_number,
            signature_algorithm=self.signature_algorithm,
            signing_key_path=self.signing_key,
            cert_type=self.type,
            time_parameters=self.time_parameters,
            use_agent=self.use_agent,
            environ_update={"TZ": "UTC"},
            check_rc=True,
        )

        temp_cert = os.path.splitext(key_copy)[0] + "-cert.pub"
        self.module.add_cleanup_file(temp_cert)

        return temp_cert

    @OpensshModule.trigger_change
    @OpensshModule.skip_if_check_mode
    def _remove(self) -> None:
        try:
            os.remove(self.path)
        except OSError as e:
            self.module.fail_json(msg=f"Unable to remove existing certificate: {e}")

    @property
    def _result(self) -> dict[str, t.Any]:
        if self.state != "present":
            return {}

        certificate_info = self.ssh_keygen.get_certificate_info(
            certificate_path=self.path,
            check_rc=self.state == "present" and not self.module.check_mode,
        )[1]

        return {
            "type": self.type,
            "filename": self.path,
            "info": format_cert_info(certificate_info),
        }

    @property
    def diff(self) -> dict[str, t.Any]:
        return {
            "before": get_cert_dict(self.original_data),
            "after": get_cert_dict(self.data),
        }


def format_cert_info(cert_info: str) -> list[str]:
    result = []
    string = ""

    for word in cert_info.split():
        if word in (
            "Type:",
            "Public",
            "Signing",
            "Key",
            "Serial:",
            "Valid:",
            "Principals:",
            "Critical",
            "Extensions:",
        ):
            result.append(string)
            string = word
        else:
            string += " " + word
    result.append(string)
    # Drop the certificate path
    result.pop(0)
    return result


def get_cert_dict(data: OpensshCertificate | None) -> dict[str, t.Any]:
    if data is None:
        return {}

    result = data.to_dict()
    result.pop("nonce")
    result["signature_algorithm"] = data.signature_type

    return result


def main() -> t.NoReturn:
    module = AnsibleModule(
        argument_spec={
            "force": {"type": "bool", "default": False},
            "identifier": {"type": "str"},
            "options": {"type": "list", "elements": "str"},
            "path": {"type": "path", "required": True},
            "pkcs11_provider": {"type": "str"},
            "principals": {"type": "list", "elements": "str"},
            "public_key": {"type": "path"},
            "regenerate": {
                "type": "str",
                "default": "partial_idempotence",
                "choices": [
                    "never",
                    "fail",
                    "partial_idempotence",
                    "full_idempotence",
                    "always",
                ],
            },
            "signature_algorithm": {
                "type": "str",
                "choices": ["ssh-rsa", "rsa-sha2-256", "rsa-sha2-512"],
            },
            "signing_key": {"type": "path"},
            "serial_number": {"type": "int"},
            "state": {
                "type": "str",
                "default": "present",
                "choices": ["absent", "present"],
            },
            "type": {"type": "str", "choices": ["host", "user"]},
            "use_agent": {"type": "bool", "default": False},
            "valid_at": {"type": "str"},
            "valid_from": {"type": "str"},
            "valid_to": {"type": "str"},
            "ignore_timestamps": {"type": "bool", "default": False},
        },
        supports_check_mode=True,
        add_file_common_args=True,
        required_if=[
            (
                "state",
                "present",
                ["type", "signing_key", "public_key", "valid_from", "valid_to"],
            )
        ],
    )

    Certificate(module).execute()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_switch_control_policy
short_description: Switch Control Policy configuration for Cisco Intersight
description:
  - Manages Switch Control Policy configuration on Cisco Intersight.
  - A policy to configure switching modes, VLAN optimization, MAC address aging, and UDLD settings for Cisco Intersight managed fabric interconnects.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/fabric/SwitchControlPolicies/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles and Policies that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Switch Control Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Switch Control Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  ethernet_switching_mode:
    description:
      - Enable or Disable Ethernet End Host Switching Mode.
      - Ethernet End Host Switching Mode is not applicable for Unified Edge; the value defaults to Ethernet Switch Mode.
      - C(end-host) - Ethernet End Host Switching Mode (default).
      - C(switch) - Ethernet Switch Mode.
    type: str
    choices: [end-host, switch]
    default: end-host
  fc_switching_mode:
    description:
      - Enable or Disable FC End Host Switching Mode.
      - FC is not supported on Unified Edge, so this setting cannot be configured and is ignored.
      - C(end-host) - FC End Host Switching Mode (default).
      - C(switch) - FC Switch Mode.
    type: str
    choices: [end-host, switch]
    default: end-host
  vlan_port_optimization_enabled:
    description:
      - To enable or disable the VLAN port count optimization.
      - This feature will always be enabled for Cisco UCS Fabric Interconnect 9108 100G.
      - Also enabled on the IMM 6.x Bundle version and onwards.
      - VLAN Port Count Optimization is not applicable for Unified Edge.
    type: bool
    default: false
  reserved_vlan_start_id:
    description:
      - The starting ID for VLANs reserved for internal use within the Fabric Interconnect.
      - This VLAN ID is the starting ID of a contiguous block of 128 VLANs that cannot be configured for user data.
      - This range of VLANs cannot be configured in VLAN policy.
      - If this property is not configured, VLAN range 3915 - 4042 is reserved for internal use by default.
      - The reserved VLAN range is fixed for Unified Edge, so this setting cannot be configured and is ignored.
    type: int
    default: 3915
  mac_aging_option:
    description:
      - MAC address aging time configuration option.
      - C(default) - Use default MAC aging time with UDLD message interval (default).
      - C(custom) - Use custom MAC aging time in seconds.
      - C(never) - MAC addresses never age out.
    type: str
    choices: [default, custom, never]
    default: default
  mac_aging_time:
    description:
      - Define the MAC address aging time in seconds.
      - This field is valid when the C(mac_aging_option) is set to C(custom).
      - Valid range is 120-918000 seconds.
    type: int
    default: 14500
  message_interval:
    description:
      - Configures the time between UDLD probe messages on the UDLD enabled ports.
      - Valid values are from 7 to 90 seconds.
    type: int
    default: 15
  recovery_action:
    description:
      - UDLD recovery action when enabled, attempts to bring an UDLD error-disabled port out of reset.
      - C(none) - No recovery action (default).
      - C(reset) - Reset the port to recover from UDLD error-disabled state.
    type: str
    choices: [none, reset]
    default: none
  fabric_pc_vhba_reset:
    description:
      - When enabled, a Registered State Change Notification (RSCN) is sent to the VIC adapter.
      - This occurs when any member port within the fabric port-channel goes down and vHBA would reset to restore the connection immediately.
      - When disabled (default), vHBA reset is done only when all the members of a fabric port-channel are down.
      - Fabric port-channel vHBA reset is not supported on Unified Edge and cannot be enabled.
      - C(enabled) - Enable fabric port-channel vHBA reset.
      - C(disabled) - Disable fabric port-channel vHBA reset (default).
    type: str
    choices: [enabled, disabled]
    default: disabled
  primary_key:
    description:
      - Encrypts MACsec keys in type-6 format.
      - If a MACsec key is already provided in a type-6 format, the primary key decrypts it.
      - MACSec is not supported on Unified Edge, so the primary key cannot be configured.
      - Must be 16-64 characters in length when specified.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create a Switch Control Policy with default settings
  cisco.intersight.intersight_switch_control_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "SwitchControl-Policy-01"
    description: "Switch control policy with default settings"
    tags:
      - Key: "Site"
        Value: "DataCenter-A"
    state: present

- name: Create a Switch Control Policy with custom settings
  cisco.intersight.intersight_switch_control_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "SwitchControl-Custom-Policy"
    description: "Switch control policy with custom MAC aging"
    ethernet_switching_mode: switch
    fc_switching_mode: switch
    vlan_port_optimization_enabled: true
    reserved_vlan_start_id: 3915
    mac_aging_option: custom
    mac_aging_time: 14500
    message_interval: 15
    recovery_action: reset
    fabric_pc_vhba_reset: enabled
    primary_key: "mySecureKey12345"
    state: present

- name: Create a Switch Control Policy with end-host mode
  cisco.intersight.intersight_switch_control_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "SwitchControl-EndHost-Policy"
    ethernet_switching_mode: end-host
    fc_switching_mode: end-host
    vlan_port_optimization_enabled: false
    recovery_action: none
    fabric_pc_vhba_reset: disabled
    state: present

- name: Create a Switch Control Policy with never aging MAC addresses
  cisco.intersight.intersight_switch_control_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "SwitchControl-NeverAge-Policy"
    mac_aging_option: never
    message_interval: 20
    state: present

- name: Update a Switch Control Policy
  cisco.intersight.intersight_switch_control_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "SwitchControl-Policy-01"
    description: "Updated switch control policy"
    ethernet_switching_mode: switch
    vlan_port_optimization_enabled: true
    state: present

- name: Delete a Switch Control Policy
  cisco.intersight.intersight_switch_control_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "SwitchControl-Policy-01"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "SwitchControl-Policy-01",
        "ObjectType": "fabric.SwitchControlPolicy",
        "EthernetSwitchingMode": "end-host",
        "FcSwitchingMode": "end-host",
        "VlanPortOptimizationEnabled": false,
        "ReservedVlanStartId": 3915,
        "MacAgingSettings": {
            "MacAgingOption": "Default"
        },
        "UdldSettings": {
            "MessageInterval": 15,
            "RecoveryAction": "none"
        },
        "FabricPcVhbaReset": "Disabled",
        "Tags": [
            {
                "Key": "Site",
                "Value": "DataCenter-A"
            }
        ]
    }
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        ethernet_switching_mode=dict(type='str', choices=['end-host', 'switch'], default='end-host'),
        fc_switching_mode=dict(type='str', choices=['end-host', 'switch'], default='end-host'),
        vlan_port_optimization_enabled=dict(type='bool', default=False),
        reserved_vlan_start_id=dict(type='int', default=3915),
        mac_aging_option=dict(type='str', choices=['default', 'custom', 'never'], default='default'),
        mac_aging_time=dict(type='int', default=14500),
        message_interval=dict(type='int', default=15),
        recovery_action=dict(type='str', choices=['none', 'reset'], default='none'),
        fabric_pc_vhba_reset=dict(type='str', choices=['enabled', 'disabled'], default='disabled'),
        primary_key=dict(type='str', no_log=True)
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure policy
    resource_path = '/fabric/SwitchControlPolicies'

    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name'],
    }

    if intersight.module.params['state'] == 'present':
        intersight.set_tags_and_description()

        # Convert lowercase choices to API format
        ethernet_mode = intersight.module.params['ethernet_switching_mode']
        intersight.api_body['EthernetSwitchingMode'] = ethernet_mode

        fc_mode = intersight.module.params['fc_switching_mode']
        intersight.api_body['FcSwitchingMode'] = fc_mode

        intersight.api_body['VlanPortOptimizationEnabled'] = intersight.module.params['vlan_port_optimization_enabled']
        intersight.api_body['ReservedVlanStartId'] = intersight.module.params['reserved_vlan_start_id']

        # Build MacAgingSettings object
        mac_aging_option = intersight.module.params['mac_aging_option']
        mac_aging_settings = {
            'MacAgingOption': mac_aging_option.capitalize()
        }
        if mac_aging_option == 'custom':
            mac_aging_settings['MacAgingTime'] = intersight.module.params['mac_aging_time']

        intersight.api_body['MacAgingSettings'] = mac_aging_settings

        # Build UdldSettings object
        intersight.api_body['UdldSettings'] = {
            'MessageInterval': intersight.module.params['message_interval'],
            'RecoveryAction': intersight.module.params['recovery_action']
        }

        # Set fabric port-channel vHBA reset
        fabric_pc_vhba_reset = intersight.module.params['fabric_pc_vhba_reset']
        intersight.api_body['FabricPcVhbaReset'] = fabric_pc_vhba_reset.capitalize()

        # Set primary key if provided
        if intersight.module.params.get('primary_key'):
            intersight.api_body['AesPrimaryKey'] = intersight.module.params['primary_key']
        else:
            intersight.api_body['AesPrimaryKey'] = ""

    # Configure the policy
    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# (c) 2023 Red Hat Inc.
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
author: Ansible Networking Team (@ansible-network)
name: default
short_description: General purpose cliconf plugin for new platforms
description:
- This plugin attemts to provide low level abstraction apis for sending and receiving CLI
  commands from arbitrary network devices.
version_added: 5.2.0
"""

import json

from ansible.errors import AnsibleConnectionFailure

from ansible_collections.ansible.netcommon.plugins.plugin_utils.cliconf_base import CliconfBase


class Cliconf(CliconfBase):
    def __init__(self, *args, **kwargs):
        super(Cliconf, self).__init__(*args, **kwargs)
        self._device_info = {}

    def get_device_info(self):
        if not self._device_info:
            device_info = {}

            device_info["network_os"] = "default"
            self._device_info = device_info

        return self._device_info

    def get_config(self, flags=None, format=None):
        network_os = self.get_device_info()["network_os"]
        raise AnsibleConnectionFailure("get_config is not supported by network_os %s" % network_os)

    def edit_config(self, candidate=None, commit=True, replace=None, comment=None):
        network_os = self.get_device_info()["network_os"]
        raise AnsibleConnectionFailure("edit_config is not supported by network_os %s" % network_os)

    def get_capabilities(self):
        result = super(Cliconf, self).get_capabilities()
        result["device_operations"] = self.get_device_operations()
        return json.dumps(result)

    def get_device_operations(self):
        return {
            "supports_diff_replace": False,
            "supports_commit": False,
            "supports_rollback": False,
            "supports_defaults": False,
            "supports_onbox_diff": False,
            "supports_commit_comment": False,
            "supports_multiline_delimiter": False,
            "supports_diff_match": False,
            "supports_diff_ignore_lines": False,
            "supports_generate_diff": False,
            "supports_replace": False,
        }



#!/usr/bin/python
# Copyright Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: bigpanda
author: "Hagai Kariti (@hkariti)"
short_description: Notify BigPanda about deployments
description:
  - Notify BigPanda when deployments start and end (successfully or not). Returns a deployment object containing all the parameters
    for future module calls.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  component:
    type: str
    description:
      - 'The name of the component being deployed. Ex: V(billing).'
    required: true
    aliases: ['name']
  version:
    type: str
    description:
      - The deployment version.
    required: true
  token:
    type: str
    description:
      - API token.
    required: true
  state:
    type: str
    description:
      - State of the deployment.
    required: true
    choices: ['started', 'finished', 'failed']
  hosts:
    type: str
    description:
      - Name of affected host name. Can be a list.
      - If not specified, it defaults to the remote system's hostname.
    aliases: ['host']
  env:
    type: str
    description:
      - The environment name, typically V(production), V(staging), and so on.
  owner:
    type: str
    description:
      - The person responsible for the deployment.
  description:
    type: str
    description:
      - Free text description of the deployment.
  url:
    type: str
    description:
      - Base URL of the API server.
    default: "https://api.bigpanda.io"
  validate_certs:
    description:
      - If V(false), SSL certificates for the target URL are not validated. This should only be used on personally controlled
        sites using self-signed certificates.
    default: true
    type: bool
  deployment_message:
    type: str
    description:
      - Message about the deployment.
    version_added: '0.2.0'
  source_system:
    type: str
    description:
      - Source system used in the requests to the API.
    default: ansible

# informational: requirements for nodes
requirements: []
"""

EXAMPLES = r"""
- name: Notify BigPanda about a deployment
  community.general.bigpanda:
    component: myapp
    version: '1.3'
    token: '{{ bigpanda_token }}'
    state: started

- name: Notify BigPanda about a deployment
  community.general.bigpanda:
    component: myapp
    version: '1.3'
    token: '{{ bigpanda_token }}'
    state: finished

# If outside servers aren't reachable from your machine, use delegate_to and override hosts:
- name: Notify BigPanda about a deployment
  community.general.bigpanda:
    component: myapp
    version: '1.3'
    token: '{{ bigpanda_token }}'
    hosts: '{{ ansible_hostname }}'
    state: started
  delegate_to: localhost
  register: deployment

- name: Notify BigPanda about a deployment
  community.general.bigpanda:
    component: '{{ deployment.component }}'
    version: '{{ deployment.version }}'
    token: '{{ deployment.token }}'
    state: finished
  delegate_to: localhost
"""

# ===========================================
# Module execution.
#
import json
import socket
import traceback

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url


def main():
    module = AnsibleModule(
        argument_spec=dict(
            component=dict(required=True, aliases=["name"]),
            version=dict(required=True),
            token=dict(required=True, no_log=True),
            state=dict(required=True, choices=["started", "finished", "failed"]),
            hosts=dict(aliases=["host"]),
            env=dict(),
            owner=dict(),
            description=dict(),
            deployment_message=dict(),
            source_system=dict(default="ansible"),
            validate_certs=dict(default=True, type="bool"),
            url=dict(default="https://api.bigpanda.io"),
        ),
        supports_check_mode=True,
    )

    token = module.params["token"]
    state = module.params["state"]
    url = module.params["url"]

    # Build the common request body
    body = dict()
    for k in ("component", "version", "hosts"):
        v = module.params[k]
        if v is not None:
            body[k] = v
    if body.get("hosts") is None:
        body["hosts"] = [socket.gethostname()]

    if not isinstance(body["hosts"], list):
        body["hosts"] = [body["hosts"]]

    # Insert state-specific attributes to body
    if state == "started":
        for k in ("source_system", "env", "owner", "description"):
            v = module.params[k]
            if v is not None:
                body[k] = v

        request_url = f"{url}/data/events/deployments/start"
    else:
        message = module.params["deployment_message"]
        if message is not None:
            body["errorMessage"] = message

        if state == "finished":
            body["status"] = "success"
        else:
            body["status"] = "failure"

        request_url = f"{url}/data/events/deployments/end"

    # Build the deployment object we return
    deployment = dict(token=token, url=url)
    deployment.update(body)
    if "errorMessage" in deployment:
        message = deployment.pop("errorMessage")
        deployment["message"] = message

    # If we're in check mode, just exit pretending like we succeeded
    if module.check_mode:
        module.exit_json(changed=True, **deployment)

    # Send the data to bigpanda
    data = json.dumps(body)
    headers = {"Authorization": f"Bearer {token}", "Content-Type": "application/json"}
    try:
        response, info = fetch_url(module, request_url, data=data, headers=headers)
        if info["status"] == 200:
            module.exit_json(changed=True, **deployment)
        else:
            module.fail_json(msg=json.dumps(info))
    except Exception as e:
        module.fail_json(msg=f"{e}", exception=traceback.format_exc())


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (C) 2019 Huawei
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

###############################################################################
# Documentation
###############################################################################

DOCUMENTATION = r"""
module: hwc_vpc_security_group_rule
description:
  - VPC security group management.
short_description: Creates a resource of VPC/SecurityGroupRule in Huawei Cloud
notes:
  - If O(id) option is provided, it takes precedence over O(security_group_id) for security group rule selection.
  - O(security_group_id) is used for security group rule selection. If more than one security group rule with this options
    exists, execution is aborted.
  - No parameter support updating. If one of option is changed, the module creates a new resource.
version_added: '0.2.0'
author: Huawei Inc. (@huaweicloud)
requirements:
  - keystoneauth1 >= 3.6.0
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  state:
    description:
      - Whether the given object should exist in Huawei Cloud.
    type: str
    choices: ['present', 'absent']
    default: 'present'
  direction:
    description:
      - Specifies the direction of access control. The value can be egress or ingress.
    type: str
    required: true
  security_group_id:
    description:
      - Specifies the security group rule ID, which uniquely identifies the security group rule.
    type: str
    required: true
  description:
    description:
      - Provides supplementary information about the security group rule. The value is a string of no more than 255 characters
        that can contain letters and digits.
    type: str
  ethertype:
    description:
      - Specifies the IP protocol version. The value can be IPv4 or IPv6. If you do not set this parameter, IPv4 is used by
        default.
    type: str
  port_range_max:
    description:
      - Specifies the end port number. The value ranges from 1 to 65535. If the protocol is not icmp, the value cannot be
        smaller than the port_range_min value. An empty value indicates all ports.
    type: int
  port_range_min:
    description:
      - Specifies the start port number. The value ranges from 1 to 65535. The value cannot be greater than the port_range_max
        value. An empty value indicates all ports.
    type: int
  protocol:
    description:
      - Specifies the protocol type. The value can be icmp, tcp, or udp. If the parameter is left blank, the security group
        supports all protocols.
    type: str
  remote_group_id:
    description:
      - Specifies the ID of the peer security group. The value is exclusive with parameter remote_ip_prefix.
    type: str
  remote_ip_prefix:
    description:
      - Specifies the remote IP address. If the access control direction is set to egress, the parameter specifies the source
        IP address. If the access control direction is set to ingress, the parameter specifies the destination IP address.
        The value can be in the CIDR format or IP addresses. The parameter is exclusive with parameter remote_group_id.
    type: str
extends_documentation_fragment:
  - community.general._hwc
  - community.general._attributes
"""

EXAMPLES = r"""
# create a security group rule
- name: Create a security group
  hwc_vpc_security_group:
    name: "ansible_network_security_group_test"
  register: sg
- name: Create a security group rule
  community.general.hwc_vpc_security_group_rule:
    direction: "ingress"
    protocol: "tcp"
    ethertype: "IPv4"
    port_range_max: 22
    security_group_id: "{{ sg.id }}"
    port_range_min: 22
    remote_ip_prefix: "0.0.0.0/0"
"""

RETURN = r"""
direction:
  description:
    - Specifies the direction of access control. The value can be egress or ingress.
  type: str
  returned: success
security_group_id:
  description:
    - Specifies the security group rule ID, which uniquely identifies the security group rule.
  type: str
  returned: success
description:
  description:
    - Provides supplementary information about the security group rule. The value is a string of no more than 255 characters
      that can contain letters and digits.
  type: str
  returned: success
ethertype:
  description:
    - Specifies the IP protocol version. The value can be IPv4 or IPv6. If you do not set this parameter, IPv4 is used by
      default.
  type: str
  returned: success
port_range_max:
  description:
    - Specifies the end port number. The value ranges from 1 to 65535. If the protocol is not icmp, the value cannot be smaller
      than the port_range_min value. An empty value indicates all ports.
  type: int
  returned: success
port_range_min:
  description:
    - Specifies the start port number. The value ranges from 1 to 65535. The value cannot be greater than the port_range_max
      value. An empty value indicates all ports.
  type: int
  returned: success
protocol:
  description:
    - Specifies the protocol type. The value can be icmp, tcp, or udp. If the parameter is left blank, the security group
      supports all protocols.
  type: str
  returned: success
remote_group_id:
  description:
    - Specifies the ID of the peer security group. The value is exclusive with parameter remote_ip_prefix.
  type: str
  returned: success
remote_ip_prefix:
  description:
    - Specifies the remote IP address. If the access control direction is set to egress, the parameter specifies the source
      IP address. If the access control direction is set to ingress, the parameter specifies the destination IP address. The
      value can be in the CIDR format or IP addresses. The parameter is exclusive with parameter remote_group_id.
  type: str
  returned: success
"""

from ansible_collections.community.general.plugins.module_utils._hwc_utils import (
    Config,
    HwcClientException,
    HwcModule,
    are_different_dicts,
    build_path,
    get_region,
    is_empty_value,
    navigate_value,
)


def build_module():
    return HwcModule(
        argument_spec=dict(
            state=dict(default="present", choices=["present", "absent"], type="str"),
            direction=dict(type="str", required=True),
            security_group_id=dict(type="str", required=True),
            description=dict(type="str"),
            ethertype=dict(type="str"),
            port_range_max=dict(type="int"),
            port_range_min=dict(type="int"),
            protocol=dict(type="str"),
            remote_group_id=dict(type="str"),
            remote_ip_prefix=dict(type="str"),
        ),
        supports_check_mode=True,
    )


def main():
    """Main function"""

    module = build_module()
    config = Config(module, "vpc")

    try:
        resource = None
        if module.params["id"]:
            resource = True
        else:
            v = search_resource(config)
            if len(v) > 1:
                raise Exception(f"Found more than one resource({', '.join([navigate_value(i, ['id']) for i in v])})")

            if len(v) == 1:
                resource = v[0]
                module.params["id"] = navigate_value(resource, ["id"])

        result = {}
        changed = False
        if module.params["state"] == "present":
            if resource is None:
                if not module.check_mode:
                    create(config)
                changed = True

            current = read_resource(config, exclude_output=True)
            expect = user_input_parameters(module)
            if are_different_dicts(expect, current):
                raise Exception(
                    f"Cannot change option from ({current}) to ({expect}) for an existing security group({module.params.get('id')})."
                )
            result = read_resource(config)
            result["id"] = module.params.get("id")
        else:
            if resource:
                if not module.check_mode:
                    delete(config)
                changed = True

    except Exception as ex:
        module.fail_json(msg=str(ex))

    else:
        result["changed"] = changed
        module.exit_json(**result)


def user_input_parameters(module):
    return {
        "description": module.params.get("description"),
        "direction": module.params.get("direction"),
        "ethertype": module.params.get("ethertype"),
        "port_range_max": module.params.get("port_range_max"),
        "port_range_min": module.params.get("port_range_min"),
        "protocol": module.params.get("protocol"),
        "remote_group_id": module.params.get("remote_group_id"),
        "remote_ip_prefix": module.params.get("remote_ip_prefix"),
        "security_group_id": module.params.get("security_group_id"),
    }


def create(config):
    module = config.module
    client = config.client(get_region(module), "vpc", "project")
    opts = user_input_parameters(module)

    params = build_create_parameters(opts)
    r = send_create_request(module, params, client)
    module.params["id"] = navigate_value(r, ["security_group_rule", "id"])


def delete(config):
    module = config.module
    client = config.client(get_region(module), "vpc", "project")

    send_delete_request(module, None, client)


def read_resource(config, exclude_output=False):
    module = config.module
    client = config.client(get_region(module), "vpc", "project")

    res = {}

    r = send_read_request(module, client)
    res["read"] = fill_read_resp_body(r)

    return update_properties(module, res, None, exclude_output)


def _build_query_link(opts):
    query_link = "?marker={marker}&limit=10"
    v = navigate_value(opts, ["security_group_id"])
    if v:
        query_link += f"&security_group_id={v}"

    return query_link


def search_resource(config):
    module = config.module
    client = config.client(get_region(module), "vpc", "project")
    opts = user_input_parameters(module)
    identity_obj = _build_identity_object(opts)
    query_link = _build_query_link(opts)
    link = f"security-group-rules{query_link}"

    result = []
    p = {"marker": ""}
    while True:
        url = link.format(**p)
        r = send_list_request(module, client, url)
        if not r:
            break

        for item in r:
            item = fill_list_resp_body(item)
            if not are_different_dicts(identity_obj, item):
                result.append(item)

        if len(result) > 1:
            break

        p["marker"] = r[-1].get("id")

    return result


def build_create_parameters(opts):
    params = dict()

    v = navigate_value(opts, ["description"], None)
    if not is_empty_value(v):
        params["description"] = v

    v = navigate_value(opts, ["direction"], None)
    if not is_empty_value(v):
        params["direction"] = v

    v = navigate_value(opts, ["ethertype"], None)
    if not is_empty_value(v):
        params["ethertype"] = v

    v = navigate_value(opts, ["port_range_max"], None)
    if not is_empty_value(v):
        params["port_range_max"] = v

    v = navigate_value(opts, ["port_range_min"], None)
    if not is_empty_value(v):
        params["port_range_min"] = v

    v = navigate_value(opts, ["protocol"], None)
    if not is_empty_value(v):
        params["protocol"] = v

    v = navigate_value(opts, ["remote_group_id"], None)
    if not is_empty_value(v):
        params["remote_group_id"] = v

    v = navigate_value(opts, ["remote_ip_prefix"], None)
    if not is_empty_value(v):
        params["remote_ip_prefix"] = v

    v = navigate_value(opts, ["security_group_id"], None)
    if not is_empty_value(v):
        params["security_group_id"] = v

    if not params:
        return params

    params = {"security_group_rule": params}

    return params


def send_create_request(module, params, client):
    url = "security-group-rules"
    try:
        r = client.post(url, params)
    except HwcClientException as ex:
        msg = f"module(hwc_vpc_security_group_rule): error running api(create), error: {ex}"
        module.fail_json(msg=msg)

    return r


def send_delete_request(module, params, client):
    url = build_path(module, "security-group-rules/{id}")

    try:
        r = client.delete(url, params)
    except HwcClientException as ex:
        msg = f"module(hwc_vpc_security_group_rule): error running api(delete), error: {ex}"
        module.fail_json(msg=msg)

    return r


def send_read_request(module, client):
    url = build_path(module, "security-group-rules/{id}")

    r = None
    try:
        r = client.get(url)
    except HwcClientException as ex:
        msg = f"module(hwc_vpc_security_group_rule): error running api(read), error: {ex}"
        module.fail_json(msg=msg)

    return navigate_value(r, ["security_group_rule"], None)


def fill_read_resp_body(body):
    result = dict()

    result["description"] = body.get("description")

    result["direction"] = body.get("direction")

    result["ethertype"] = body.get("ethertype")

    result["id"] = body.get("id")

    result["port_range_max"] = body.get("port_range_max")

    result["port_range_min"] = body.get("port_range_min")

    result["protocol"] = body.get("protocol")

    result["remote_address_group_id"] = body.get("remote_address_group_id")

    result["remote_group_id"] = body.get("remote_group_id")

    result["remote_ip_prefix"] = body.get("remote_ip_prefix")

    result["security_group_id"] = body.get("security_group_id")

    return result


def update_properties(module, response, array_index, exclude_output=False):
    r = user_input_parameters(module)

    v = navigate_value(response, ["read", "description"], array_index)
    r["description"] = v

    v = navigate_value(response, ["read", "direction"], array_index)
    r["direction"] = v

    v = navigate_value(response, ["read", "ethertype"], array_index)
    r["ethertype"] = v

    v = navigate_value(response, ["read", "port_range_max"], array_index)
    r["port_range_max"] = v

    v = navigate_value(response, ["read", "port_range_min"], array_index)
    r["port_range_min"] = v

    v = navigate_value(response, ["read", "protocol"], array_index)
    r["protocol"] = v

    v = navigate_value(response, ["read", "remote_group_id"], array_index)
    r["remote_group_id"] = v

    v = navigate_value(response, ["read", "remote_ip_prefix"], array_index)
    r["remote_ip_prefix"] = v

    v = navigate_value(response, ["read", "security_group_id"], array_index)
    r["security_group_id"] = v

    return r


def send_list_request(module, client, url):
    r = None
    try:
        r = client.get(url)
    except HwcClientException as ex:
        msg = f"module(hwc_vpc_security_group_rule): error running api(list), error: {ex}"
        module.fail_json(msg=msg)

    return navigate_value(r, ["security_group_rules"], None)


def _build_identity_object(all_opts):
    result = dict()

    v = navigate_value(all_opts, ["description"], None)
    result["description"] = v

    v = navigate_value(all_opts, ["direction"], None)
    result["direction"] = v

    v = navigate_value(all_opts, ["ethertype"], None)
    result["ethertype"] = v

    result["id"] = None

    v = navigate_value(all_opts, ["port_range_max"], None)
    result["port_range_max"] = v

    v = navigate_value(all_opts, ["port_range_min"], None)
    result["port_range_min"] = v

    v = navigate_value(all_opts, ["protocol"], None)
    result["protocol"] = v

    result["remote_address_group_id"] = None

    v = navigate_value(all_opts, ["remote_group_id"], None)
    result["remote_group_id"] = v

    v = navigate_value(all_opts, ["remote_ip_prefix"], None)
    result["remote_ip_prefix"] = v

    v = navigate_value(all_opts, ["security_group_id"], None)
    result["security_group_id"] = v

    return result


def fill_list_resp_body(body):
    result = dict()

    result["description"] = body.get("description")

    result["direction"] = body.get("direction")

    result["ethertype"] = body.get("ethertype")

    result["id"] = body.get("id")

    result["port_range_max"] = body.get("port_range_max")

    result["port_range_min"] = body.get("port_range_min")

    result["protocol"] = body.get("protocol")

    result["remote_address_group_id"] = body.get("remote_address_group_id")

    result["remote_group_id"] = body.get("remote_group_id")

    result["remote_ip_prefix"] = body.get("remote_ip_prefix")

    result["security_group_id"] = body.get("security_group_id")

    return result


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage Check Point Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

DOCUMENTATION = """
---
module: cp_mgmt_user
short_description: Manages user objects on Checkpoint over Web Services API
description:
  - Manages user objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R80.40 JHF management version.
version_added: "6.3.0"
author: "Dor Berenstein (@chkp-dorbe)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  template:
    description:
      - User template name or UID.
    type: str
  email:
    description:
      - User email.
    type: str
  expiration_date:
    description:
      - Expiration date in format, yyyy-MM-dd.
    type: str
  phone_number:
    description:
      - User phone number.
    type: str
  authentication_method:
    description:
      - Authentication method.
    type: str
    choices: ['undefined', 'check point password', 'os password', 'securid', 'radius', 'tacacs']
  password:
    description:
      - Check Point password authentication method identified by the name or UID. Must be set when "authentication-method" was selected to be "Check
        Point Password".
    type: str
  radius_server:
    description:
      - RADIUS server object identified by the name or UID. Must be set when "authentication-method" was selected to be "RADIUS".
    type: str
  tacacs_server:
    description:
      - TACACS server object identified by the name or UID. Must be set when "authentication-method" was selected to be "TACACS".
    type: str
  connect_on_days:
    description:
      - Days users allow to connect.
    type: list
    elements: str
  connect_daily:
    description:
      - Connect every day.
    type: bool
  from_hour:
    description:
      - Allow users connect from hour.
    type: str
  to_hour:
    description:
      - Allow users connect until hour.
    type: str
  allowed_locations:
    description:
      - User allowed locations.
    type: dict
    suboptions:
      destinations:
        description:
          - Collection of allowed destination locations name or uid.
        type: list
        elements: str
      sources:
        description:
          - Collection of allowed source locations name or uid.
        type: list
        elements: str
  encryption:
    description:
      - User encryption. Doesn't support shared secret.
    type: dict
    suboptions:
      ike:
        description:
          - Enable IKE encryption for users.
        type: bool
      public_key:
        description:
          - Enable IKE public key.
        type: bool
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  groups:
    description:
      - Collection of group identifiers.
    type: list
    elements: str
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-user
  cp_mgmt_user:
    authentication_method: securid
    connect_daily: 'True'
    email: myuser@email.com
    encryption:
      enable_ike: 'True'
      enable_public_key: 'True'
    expiration_date: '2030-05-30'
    from_hour: 08:00
    name: myuser
    phone_number: '0501112233'
    state: present
    to_hour: '17:00'

- name: set-user
  cp_mgmt_user:
    authentication_method: undefined
    expiration_date: '2035-01-15'
    from_hour: '12:00'
    name: myuser
    state: present

- name: delete-user
  cp_mgmt_user:
    name: myuser
    state: absent
"""

RETURN = """
cp_mgmt_user:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, \
    api_call


def main():
    argument_spec = dict(
        name=dict(type='str', required=True),
        template=dict(type='str'),
        email=dict(type='str'),
        expiration_date=dict(type='str'),
        phone_number=dict(type='str'),
        authentication_method=dict(type='str',
                                   choices=['undefined',
                                            'check point password',
                                            'os password',
                                            'securid',
                                            'radius',
                                            'tacacs']),
        password=dict(type='str', no_log=True),
        radius_server=dict(type='str'),
        tacacs_server=dict(type='str'),
        connect_on_days=dict(type='list', elements="str"),
        connect_daily=dict(type='bool'),
        from_hour=dict(type='str'),
        to_hour=dict(type='str'),
        allowed_locations=dict(type='dict', options=dict(
            destinations=dict(type='list', elements="str"),
            sources=dict(type='list', elements="str")
        )),
        encryption=dict(type='dict', options=dict(
            ike=dict(type='bool'),
            public_key=dict(type='bool')
        )),
        color=dict(type='str', choices=['aquamarine',
                                        'black',
                                        'blue',
                                        'crete blue',
                                        'burlywood',
                                        'cyan',
                                        'dark green',
                                        'khaki',
                                        'orchid',
                                        'dark orange',
                                        'dark sea green',
                                        'pink',
                                        'turquoise',
                                        'dark blue',
                                        'firebrick',
                                        'brown',
                                        'forest green',
                                        'gold',
                                        'dark gold',
                                        'gray',
                                        'dark gray',
                                        'light green',
                                        'lemon chiffon',
                                        'coral',
                                        'sea green',
                                        'sky blue',
                                        'magenta',
                                        'purple',
                                        'slate blue',
                                        'violet red',
                                        'navy blue',
                                        'olive',
                                        'orange',
                                        'red',
                                        'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        ignore_warnings=dict(type='bool'),
        groups=dict(type='list', elements="str"),
        ignore_errors=dict(type='bool'),
        tags=dict(type='list', elements="str")
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'user'

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The eos_logging_global config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.facts import Facts
from ansible_collections.arista.eos.plugins.module_utils.network.eos.rm_templates.logging_global import (
    Logging_globalTemplate,
)


class Logging_global(ResourceModule):
    """
    The eos_logging_global config class
    """

    def __init__(self, module):
        super(Logging_global, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="logging_global",
            tmplt=Logging_globalTemplate(),
        )
        self.parsers = [
            "buffered",
            "event",
            "facility",
            "console",
            "format",
            "format.timestamp.traditional",
            "format.timestamp.highresolution",
            "level",
            "monitor",
            "on",
            "persistent",
            "policy",
            "relogging_interval",
            "repeat_messages",
            "src_interface",
            "synchronous",
            "trap",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """

        wantd = {"logging_global": self.want}
        haved = {"logging_global": self.have}
        # turn all lists of dicts into dicts prior to merge
        for entry in wantd["logging_global"], haved["logging_global"]:
            self._logging_global_list_to_dict(entry)
        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state == "deleted":
            wantd = {}
            for k, have in haved.items():
                self._compare(want={}, have=have)

        for k, want in wantd.items():
            self._compare(want=want, have=haved.pop(k, {}))

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Logging_global network resource.
        """
        self._hosts_compare(want=want, have=have)
        self._vrfs_compare(want=want, have=have)
        self.compare(parsers=self.parsers, want=want, have=have)

    def _hosts_compare(self, want, have):
        host_want = want.pop("hosts", {})
        host_have = have.pop("hosts", {})
        for name, entry in host_want.items():
            h = {}
            if host_have:
                h = {"hosts": host_have.pop(name, {})}
            self.compare(parsers="host", want={"hosts": entry}, have=h)
        for name, entry in host_have.items():
            self.compare(parsers="host", want={}, have={"hosts": entry})

    def _vrfs_hosts_compare(self, vrf, want, have):
        host_want = want.pop("hosts", {})
        host_have = have.pop("hosts", {})
        for name, entry in host_want.items():
            h = {}
            if host_have:
                h = {"vrfs": {"name": vrf, "hosts": host_have.pop(name, {})}}
            w = {"vrfs": {"name": vrf, "hosts": entry}}
            self.compare(parsers="vrf.host", want=w, have=h)
        for name, entry in host_have.items():
            self.compare(
                parsers="vrf.host",
                want={},
                have={"vrfs": {"name": vrf, "hosts": entry}},
            )

    def _vrfs_compare(self, want, have):
        vrf_want = want.pop("vrfs", {})
        vrf_have = have.pop("vrfs", {})
        for name, entry in vrf_want.items():
            self._vrfs_hosts_compare(
                name,
                want=entry,
                have=vrf_have.get(name, {}),
            )
            if entry.get("source_interface"):
                h = {}
                if vrf_have.get(name):
                    h = {
                        "vrfs": {
                            "name": name,
                            "source_interface": vrf_have[name].pop(
                                "source_interface",
                                "",
                            ),
                        },
                    }
                w = {
                    "vrfs": {
                        "name": name,
                        "source_interface": entry["source_interface"],
                    },
                }
                self.compare(parsers="vrf.source_interface", want=w, have=h)
        for name, entry in vrf_have.items():
            self._vrfs_hosts_compare(name, want={}, have=entry)
            self.compare(
                parsers="vrf.source_interface",
                want={},
                have={"vrfs": entry},
            )

    def _logging_global_list_to_dict(self, entry):
        if "hosts" in entry:
            hosts_dict = {}
            for el in entry["hosts"]:
                hosts_dict.update({el["name"]: el})
            entry["hosts"] = hosts_dict

        if "vrfs" in entry:
            vrf_dict = {}
            for el in entry["vrfs"]:
                vrf_dict.update({el["name"]: el})
            entry["vrfs"] = vrf_dict
            for k, v in entry["vrfs"].items():
                self._logging_global_list_to_dict(v)



#
# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The cisco.ios_bgp_address_family config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.facts import Facts
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.bgp_address_family import (
    Bgp_address_familyTemplate,
)


class Bgp_address_family(ResourceModule):
    """
    The cisco.ios_bgp_address_family config class
    """

    parsers = [
        "advertise",
        "as_number",  # generic
        "aggregate_addresses",
        "auto_summary",
        "default",
        "default_information",
        "default_metric",
        "distance",
        "maximum_paths.paths",
        "maximum_paths.eibgp",
        "maximum_paths.ibgp",
        "maximum_secondary_paths.paths",
        "maximum_secondary_paths.eibgp",
        "maximum_secondary_paths.ibgp",
        "table_map",
        "bgp.additional_paths.select",  # bgp
        "bgp.additional_paths.install",
        "bgp.additional_paths.receive",
        "bgp.additional_paths.send",
        "bgp.aggregate_timer",
        "bgp.dmzlink_bw",
        "bgp.nexthop.route_map",
        "bgp.nexthop.trigger.delay",
        "bgp.nexthop.trigger.enable",
        "bgp.redistribute_internal",
        "bgp.route_map",
        "bgp.scan_time",
        "bgp.soft_reconfig_backup",
        "bgp.update_group",
        "bgp.dampening",
        "bgp.slow_peer_options.detection.enable",
        "bgp.slow_peer_options.detection.threshold",
        "bgp.slow_peer_options.split_update_group.dynamic",
        "bgp.slow_peer.split_update_group.permanent",
        "snmp.context.user",  # snmp
        "snmp.context.community",
        "redistribute.application",  # redistribute
        "redistribute.bgp",
        "redistribute.connected",
        "redistribute.eigrp",
        "redistribute.isis",
        "redistribute.iso_igrp",
        "redistribute.lisp",
        "redistribute.mobile",
        "redistribute.odr",
        "redistribute.ospf",
        "redistribute.ospfv3",
        "redistribute.rip",
        "redistribute.static",
        "redistribute.vrf",  # redistribute
    ]

    def __init__(self, module):
        super(Bgp_address_family, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="bgp_address_family",
            tmplt=Bgp_address_familyTemplate(),
        )

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        if self.state in ["merged", "replaced", "overridden"]:
            if self.have.get("as_number") and self.want.get("as_number") != self.have.get(
                "as_number",
            ):
                self._module.fail_json(
                    msg="BGP is already running. Only one BGP instance is allowed per device.",
                )

        for each in self.want, self.have:
            each["address_family"] = self._bgp_add_fam_list_to_dict(each.get("address_family", []))

        wantd = self.want
        haved = self.have

        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        if self.state in ["overridden", "replaced"]:
            all_neighbors_to_delete = set()

            for af_key, have_af_val in haved.get("address_family", {}).items():
                want_af_val = wantd.get("address_family", {}).get(af_key)

                if self.state == "overridden" and want_af_val is None:
                    all_neighbors_to_delete.update(have_af_val.get("neighbors", {}).keys())
                    continue

                if want_af_val and "neighbors" in want_af_val:
                    have_neighbors = set(have_af_val.get("neighbors", {}).keys())
                    want_neighbors = set(want_af_val.get("neighbors", {}).keys())
                    neighbors_to_remove_from_af = have_neighbors - want_neighbors
                    all_neighbors_to_delete.update(neighbors_to_remove_from_af)

            for neighbor_addr in all_neighbors_to_delete:
                self.addcmd({"neighbor_address": neighbor_addr}, "neighbor_address", True)

        if self.state == "deleted":
            for k, have in haved.get("address_family", {}).items():
                if wantd.get("address_family"):
                    if wantd["address_family"].get(k):
                        self.commands.append(
                            self._tmplt.render(wantd["address_family"].get(k, {}), "afi", True),
                        )
                else:  # to clear off all afs
                    self.commands.append(self._tmplt.render(have, "afi", True))

        # remove superfluous config
        if self.state in ["overridden"]:
            for k, have in haved.get("address_family").items():
                if k not in wantd.get("address_family", {}):
                    self._compare(want={}, have=have)

        if self.state != "deleted":  # not deleted state
            for k, want in wantd.get("address_family", {}).items():
                self._compare(want=want, have=haved["address_family"].pop(k, {}))

        # adds router bgp AS_NUMB command
        if len(self.commands) > 0:
            if self.want.get("as_number"):
                as_number = self.want
            else:
                as_number = self.have
            self.commands.insert(0, self._tmplt.render(as_number, "as_number", False))

    def _compare(self, want, have):
        begin = len(self.commands)
        # for everything else
        self.compare(parsers=self.parsers, want=want, have=have)
        # for neighbors
        self._compare_neighbor_lists(want.get("neighbors", {}), have.get("neighbors", {}))
        # for networks
        self._compare_network_lists(want.get("networks", {}), have.get("networks", {}))
        # for aggregate_addresses
        self._compare_agg_add_lists(
            want.get("aggregate_addresses", {}),
            have.get("aggregate_addresses", {}),
        )
        # for distribution of ospfv2 and ospfv3 routes
        for ospf_version in ["ospf", "ospfv3"]:
            self._compare_redist_ospf(
                ospf_version,
                want.get("redistribute", {}).get(ospf_version, {}),
                have.get("redistribute", {}).get(ospf_version, {}),
            )
        # add af command
        if len(self.commands) != begin:
            self.commands.insert(begin, self._tmplt.render(want or have, "afi", False))
            self.commands.append("exit-address-family")

    def _compare_redist_ospf(self, _parser, w_attr, h_attr):
        """
        Adds and/or removes commands related to
        ospf and ospv3 redistribution
        :param _parser: ospf or ospfv3
        :param w_attr: content of want['redistribute']['ospf']
        :param h_attr:content of have['redistribute']['ospf']
        :return: None
        """
        for wkey, wentry in w_attr.items():
            if wentry != h_attr.pop(wkey, {}):
                # always negate the command in the
                # appropriate states before applying
                if self.state in ["overridden", "replaced"]:
                    self.addcmd(wentry, f"redistribute.{_parser}", True)
                self.addcmd(wentry, f"redistribute.{_parser}", False)

        # remove remaining items in have for replaced state
        for hkey, hentry in h_attr.items():
            self.addcmd(hentry, "redistribute.ospf", True)

    def _compare_neighbor_lists(self, want, have):
        """Compare neighbor list of dict"""
        neig_parses = [
            "peer_group",
            "peer_group_name",
            "local_as",
            "remote_as",
            "activate",
            "additional_paths",
            "advertises.additional_paths",
            "advertises.best_external",
            "advertises.diverse_path",
            "advertise_map",
            "advertisement_interval",
            "aigp",
            "aigp.send.cost_community",
            "aigp.send.med",
            "allow_policy",
            "allowas_in",
            "as_override",
            "bmp_activate",
            "capability",
            "cluster_id",
            "default_originate",
            "default_originate.route_map",
            "description",
            "disable_connected_check",
            "ebgp_multihop",
            "distribute_list",
            "dmzlink_bw",
            "filter_list",
            "fall_over.bfd",
            "fall_over.route_map",
            "ha_mode",
            "inherit",
            "internal_vpn_client",
            "log_neighbor_changes",
            "maximum_prefix",
            "nexthop_self.set",
            "nexthop_self.all",
            "next_hop_unchanged.set",
            "next_hop_unchanged.allpaths",
            "password_options",
            "path_attribute.discard",
            "path_attribute.treat_as_withdraw",
            "route_maps",
            "remove_private_as.set",
            "remove_private_as.all",
            "remove_private_as.replace_as",
            "route_reflector_client",
            "route_server_client",
            "send_community.set",
            "send_community.both",
            "send_community.extended",
            "send_community.standard",
            "shutdown",
            "slow_peer_options.detection",
            "slow_peer_options.split_update_group",
            "soft_reconfiguration",
            "soo",
            "timers",
            "transport.connection_mode",
            "transport.multi_session",
            "transport.path_mtu_discovery",
            "ttl_security",
            "unsuppress_map",
            "version",
            "weight",
        ]

        for name, w_neighbor in want.items():
            have_nbr = have.pop(name, {})
            self.compare(parsers=neig_parses, want=w_neighbor, have=have_nbr)
            for i in ["route_maps", "prefix_lists"]:  # handles route_maps, prefix_lists
                want_route_or_prefix = w_neighbor.pop(i, {})
                have_route_or_prefix = have_nbr.pop(i, {})
                if want_route_or_prefix:
                    for k_rmps, w_rmps in want_route_or_prefix.items():
                        have_rmps = have_route_or_prefix.pop(k_rmps, {})
                        w_rmps["neighbor_address"] = w_neighbor.get("neighbor_address")
                        if have_rmps:
                            have_rmps["neighbor_address"] = have_nbr.get("neighbor_address")
                            have_rmps = {i: have_rmps}
                        self.compare(parsers=[i], want={i: w_rmps}, have=have_rmps)

    def _compare_network_lists(self, w_attr, h_attr):
        """Handling of network list options."""
        for wkey, wentry in w_attr.items():
            if wentry != h_attr.pop(wkey, {}):
                self.addcmd(wentry, "networks", False)

        # remove remaining items in have for replaced state
        for hkey, hentry in h_attr.items():
            self.addcmd(hentry, "networks", True)

    def _compare_agg_add_lists(self, w_attr, h_attr):
        """Handling of agg_add list options."""
        for wkey, wentry in w_attr.items():
            if wentry != h_attr.pop(wkey, {}):
                self.addcmd(wentry, "aggregate_addresses", False)
        # remove remaining items in have for replaced state
        for hkey, hentry in h_attr.items():
            self.addcmd(hentry, "aggregate_addresses", True)

    def _bgp_add_fam_list_to_dict(self, tmp_data):
        """Convert all list of dicts to dicts of dicts, also deals with deprecated attributes"""
        p_key = {
            "aggregate_address": "address",
            "aggregate_addresses": "address",
            "neighbor": "neighbor_address",
            "neighbors": "neighbor_address",
            "route_maps": "name",
            "prefix_lists": "name",
            "networks": "address",
            "network": "address",
            "ospf": "process_id",
            "ospfv3": "process_id",
        }

        af_data = {}
        for af in tmp_data:
            if af.get("afi") in ["ipv4", "ipv6"] and not af.get("safi"):
                af["safi"] = "unicast"
            _af = {}
            for k, tval in af.items():
                val = deepcopy(tval)
                if k == "neighbor" or k == "neighbors":
                    tmp = {}
                    for neib in val:
                        # address/ tag/ ipv6_address to neighbor_address
                        if neib.get("address"):
                            neib["neighbor_address"] = neib.pop("address")
                        if neib.get("tag"):
                            neib["neighbor_address"] = neib.pop("tag")
                        if neib.get("ipv6_address"):
                            neib["neighbor_address"] = neib.pop("ipv6_address")
                        if neib.get("ipv6_adddress"):
                            neib["neighbor_address"] = neib.pop("ipv6_adddress")
                        # prefix_list and prefix_lists
                        if neib.get("prefix_list"):  # deprecated made list
                            neib["prefix_lists"] = [neib.pop("prefix_list")]
                        if neib.get("prefix_lists"):
                            neib["prefix_lists"] = {
                                str(i[p_key["prefix_lists"]]): i for i in neib.get("prefix_lists")
                            }
                        # route_map and route_maps
                        if neib.get("route_map"):  # deprecated made list
                            neib["route_maps"] = [neib.pop("route_map")]
                        if neib.get("route_maps"):
                            neib["route_maps"] = {
                                str(i[p_key["route_maps"]]): i for i in neib.get("route_maps")
                            }
                        # slow_peer to slow_peer_options
                        if neib.get("slow_peer"):  # only one slow_peer is allowed
                            neib["slow_peer_options"] = neib.pop("slow_peer")[0]
                        # we can skip deprecating these by handling the int to str conversion here
                        # int to float is not considered considering the size of as numbers
                        if neib.get("remote_as"):
                            neib["remote_as"] = str(neib.get("remote_as"))
                        if neib.get("local_as") and neib.get("local_as", {}).get("number"):
                            neib["local_as"]["number"] = str(neib["local_as"]["number"])
                        # make dict neighbors dict
                        tmp[neib[p_key[k]]] = neib
                    _af["neighbors"] = tmp
                # make dict networks dict
                elif k == "network" or k == "networks":
                    _af["networks"] = {str(i[p_key[k]]): i for i in tval}
                # make dict aggregate_addresses dict
                elif k == "aggregate_address" or k == "aggregate_addresses":
                    _af["aggregate_addresses"] = {str(i[p_key[k]]): i for i in tval}
                # slow_peer to slow_peer_options
                elif k == "bgp":
                    _af["bgp"] = val
                    if val.get("slow_peer"):  # only one slow_peer is allowed
                        _af["bgp"]["slow_peer_options"] = _af["bgp"].pop("slow_peer")[0]
                # keep single dict to compare redistribute
                elif k == "redistribute":
                    _redist = {}
                    for i in tval:
                        # establish elif sections for future protocols
                        # if necessary
                        if any(x in ["ospf", "ospfv3"] for x in i):
                            for ospf_version in ["ospf", "ospfv3"]:
                                if i.get(ospf_version):
                                    _i = i[ospf_version]

                                    # Start handle deprecates
                                    if _i.get("match"):
                                        for depr in [
                                            "external",
                                            "nssa_external",
                                            "type_1",
                                            "type_2",
                                        ]:
                                            if depr in _i["match"].keys():
                                                val = _i["match"].pop(depr, False)
                                                if depr.startswith("type"):
                                                    # map deprecated nssa_external type to new option
                                                    if "nssa_externals" in _i["match"].keys():
                                                        _i["match"]["nssa_externals"][depr] = val
                                                    else:
                                                        _i["match"]["nssa_externals"] = {
                                                            depr: val,
                                                        }
                                                elif depr in ["external", "nssa_external"]:
                                                    # deprecated external and nssa_external are boolean
                                                    # so both types mapped to true
                                                    _i["match"][depr + "s"] = {
                                                        "type_1": True,
                                                        "type_2": True,
                                                    }
                                    # End handle deprecates

                                    if ospf_version not in _redist:
                                        _redist[ospf_version] = {}

                                    _redist[ospf_version].update(
                                        {
                                            str(_i[p_key[ospf_version]]): dict(_i.items()),
                                        },
                                    )
                                    break
                        else:
                            _redist.update(i)
                    _af["redistribute"] = _redist
                else:
                    _af[k] = tval
            # make distinct address family entires
            af_data[af.get("afi", "") + "_" + af.get("safi", "") + "_" + af.get("vrf", "")] = _af

        return af_data



# (c) 2020, Alexei Znamensky <russoz@gmail.com>
# Copyright (c) 2020, Ansible Project
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

# pylint: disable=unused-import
from ansible_collections.community.general.plugins.module_utils._mh.deco import (  # noqa: F401
    cause_changes,
    check_mode_skip,
    check_mode_skip_returns,
    module_fails_on_exception,
)
from ansible_collections.community.general.plugins.module_utils._mh.exceptions import (
    ModuleHelperException,  # noqa: F401
)
from ansible_collections.community.general.plugins.module_utils._mh.module_helper import (  # noqa: F401
    ModuleHelper,
    StateModuleHelper,
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2022, Tim Cragg (@timcragg)
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_dns_domain
short_description: Manage DNS Provider objects (dns:Domain)
description:
- Manage DNS Domain configuration on Cisco ACI fabrics.
options:
  dns_profile:
    description:
    - Name of the DNS profile.
    type: str
    aliases: [ profile_name ]
    required: true
  domain:
    description:
    - DNS domain name
    type: str
    aliases: [ name, domain_name ]
  default:
    description:
    - Whether this is the default domain
    type: bool
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- The C(dns_profile) must exist before using this module in your playbook.
  The M(cisco.aci.aci_dns_profile) modules can be used for this.

seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(dns:Domain).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Tim Cragg (@timcragg)
"""

EXAMPLES = r"""
- name: Add a new DNS domain
  cisco.aci.aci_dns_domain:
    host: apic
    username: admin
    password: SomeSecretPassword
    dns_profile: my_dns_prof
    domain: example.com
    state: present
  delegate_to: localhost

- name: Remove a DNS domain
  cisco.aci.aci_dns_domain:
    host: apic
    username: admin
    password: SomeSecretPassword
    dns_profile: my_dns_prof
    domain: example.com
    state: absent
  delegate_to: localhost

- name: Query a DNS domain
  cisco.aci.aci_dns_domain:
    host: apic
    username: admin
    password: SomeSecretPassword
    dns_profile: my_dns_prof
    domain: example.com
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all DNS domains within a DNS profile
  cisco.aci.aci_dns_domain:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
   current:
     description: The existing configuration from the APIC after the module has finished
     returned: success
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production environment",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   error:
     description: The error information as returned from the APIC
     returned: failure
     type: dict
     sample:
       {
           "code": "122",
           "text": "unknown managed object class foo"
       }
   raw:
     description: The raw output returned by the APIC REST API (xml or json)
     returned: parse error
     type: str
     sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class "/></imdata>'
   sent:
     description: The actual/minimal configuration pushed to the APIC
     returned: info
     type: list
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment"
               }
           }
       }
   previous:
     description: The original configuration from the APIC before the module has started
     returned: info
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   proposed:
     description: The assembled configuration from the user-provided parameters
     returned: info
     type: dict
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment",
                   "name": "production"
               }
           }
       }
   filter_string:
     description: The filter string used for the request
     returned: failure or debug
     type: str
     sample: ?rsp-prop-include=config-only
   method:
     description: The HTTP method used for the request to the APIC
     returned: failure or debug
     type: str
     sample: POST
   response:
     description: The HTTP response from the APIC
     returned: failure or debug
     type: str
     sample: OK (30 bytes)
   status:
     description: The HTTP status from the APIC
     returned: failure or debug
     type: int
     sample: 200
   url:
     description: The HTTP url used for the request to the APIC
     returned: failure or debug
     type: str
     sample: https://10.11.12.13/api/mo/uni/tn-production.json
   """

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        dns_profile=dict(type="str", aliases=["profile_name"], required=True),
        domain=dict(type="str", aliases=["name", "domain_name"]),
        default=dict(type="bool"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["domain"]],
            ["state", "present", ["domain"]],
        ],
    )

    aci = ACIModule(module)

    dns_profile = module.params.get("dns_profile")
    domain = module.params.get("domain")
    default = aci.boolean(module.params.get("default"))
    state = module.params.get("state")

    aci.construct_url(
        root_class=dict(
            aci_class="dnsProfile",
            aci_rn="fabric/dnsp-{0}".format(dns_profile),
            module_object=dns_profile,
            target_filter={"name": dns_profile},
        ),
        subclass_1=dict(aci_class="dnsDomain", aci_rn="dom-{0}".format(domain), module_object=domain, target_filter={"name": domain}),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="dnsDomain",
            class_config=dict(name=domain, isDefault=default),
        )

        aci.get_diff(aci_class="dnsDomain")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2018 Fred-sun, <xiuxi.sun@qq.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_datafactory

version_added: "0.1.12"

short_description: Managed data factory

description:
    - Create, update or delete data factory.

options:
    name:
        description:
            - The factory name.
        type: str
        required: true
    resource_group:
        description:
            - Limit results by resource group. Required when using name parameter.
        type: str
        required: true
    if_match:
        description:
            - ETag of the factory entity.
            - Should only be specified for get.
            - If the ETag matches the existing entity tag, or if * was provided, then no content will be returned.
        type: str
    repo_configuration:
        description:
            - The data factory repo configration.
        type: dict
        suboptions:
            type:
                description:
                    - Type of repo configuration.
                type: str
                required: True
                choices:
                    - FactoryGitHubConfiguration
                    - FactoryVSTSConfiguration
            account_name:
                description:
                    - Account name.
                type: str
                required: True
            collaboration_branch:
                description:
                    - Collaboration branch.
                type: str
                required: True
            root_folder:
                description:
                    - Root folder.
                type: str
                required: True
            repository_name:
                description:
                    - Repository name.
                type: str
                required: True
            project_name:
                description:
                    - VSTS project name.
                    - Required when I(type=FactoryVSTSConfiguration).
                type: str
    location:
        description:
            - Valid Azure location. Defaults to location of the resource group.
        type: str
    public_network_access:
        description:
            - Whether or not public network access is allowed for the data factory.
        type: str
        choices:
            - Enabled
            - Disabled
    state:
        description:
            - Assert the state of the Public IP. Use C(present) to create or update a and C(absent) to delete.
        default: present
        choices:
            - absent
            - present
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Fred-sun (@Fred-sun)
    - xuzhang3 (@xuzhang3)
'''

EXAMPLES = '''
- name: Create the data factory
  azure_rm_datafactory:
    resource_group: "{{ resource_group }}"
    name: "{{ name }}"
    repo_configuration:
      type: FactoryGitHubConfiguration
      account_name: Fred-sun
      collaboration_branch: testbranch
      root_folder: "./"
      repository_name: vault
'''

RETURN = '''
state:
    description:
        - Current state fo the data factory.
    returned: always
    type: complex
    contains:
        id:
            description:
                - The data facotry ID.
            type: str
            returned: always
            sample: "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.DataFactory/factories/testpro"
        create_time:
            description:
                - Time the factory was created in ISO8601 format.
            type: str
            returned: always
            sample: "2022-04-26T08:24:41.391164+00:00"
        location:
            description:
                - The resource location.
            type: str
            returned: always
            sample: eastus
        name:
            description:
                - The resource name.
            type: str
            returned: always
            sample: testfactory
        provisioning_state:
            description:
                - Factory provisioning state, example Succeeded.
            type: str
            returned: always
            sample: Succeeded
        e_tag:
            description:
                - Etag identifies change in the resource.
            type: str
            returned: always
            sample: "3000fa80-0000-0100-0000-6267ac490000"
        type:
            description:
                - The resource type.
            type: str
            returned: always
            sample: "Microsoft.DataFactory/factories"
        public_network_access:
            description:
                - Whether or not public network access is allowed for the data factory.
            type: str
            returned: always
            sample: "Enabled"
        tags:
            description:
                - List the data factory tags.
            type: str
            returned: always
            sample: {'key1':'value1'}
        identity:
            description:
                -  Managed service identity of the factory.
            type: str
            returned: always
            contains:
                principal_id:
                    description:
                        - The principal id of the identity.
                    type: str
                    returned: always
                    sample: "***********"
                tenant_id:
                    description:
                        - The client tenant id of the identity.
                    type: str
                    returned: always
                    sample: "***********"
        repo_configuration:
            description:
                - Git repo information of the factory.
            type: str
            returned: always
            contains:
                type:
                    description:
                        - Type of repo configuration.
                    type: str
                    returned: always
                    sample: FactoryGitHubConfiguration
                ccount_name:
                    description:
                        - Account name.
                    type: str
                    returned: always
                    sample: fredaccount
                collaboration_branch:
                    description:
                        - Collaboration branch.
                    type: str
                    returned: always
                    sample: branch
                repository_name:
                    description:
                        - Repository name.
                    type: str
                    returned: always
                    sample:  "vault"
                root_folder:
                    description:
                        - Root folder.
                    type: str
                    returned: always
                    sample: "/home/"
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

AZURE_OBJECT_CLASS = 'DataFactory'

repo_configuration_spec = dict(
    type=dict(type='str', required=True, choices=['FactoryVSTSConfiguration', 'FactoryGitHubConfiguration']),
    account_name=dict(type='str', required=True),
    repository_name=dict(type='str', required=True),
    collaboration_branch=dict(type='str', required=True),
    root_folder=dict(type='str', required=True),
    project_name=dict(type='str'),
)


class AzureRMDataFactory(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str', required=True),
            resource_group=dict(type='str', required=True),
            if_match=dict(type='str'),
            location=dict(type='str'),
            public_network_access=dict(type='str', choices=["Enabled", "Disabled"]),
            state=dict(type='str', default='present', choices=['absent', 'present']),
            repo_configuration=dict(type='dict', options=repo_configuration_spec),
        )

        self.results = dict(
            changed=False,
        )

        self.name = None
        self.resource_group = None
        self.if_match = None
        self.location = None
        self.tags = None
        self.public_network_access = None
        self.repo_configuration = None

        super(AzureRMDataFactory, self).__init__(self.module_arg_spec,
                                                 supports_check_mode=True,
                                                 supports_tags=True,
                                                 facts_module=False)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            self.location = resource_group.location

        response = self.get_item()
        changed = False
        if self.state == 'present':
            if response:
                if self.tags is not None:
                    update_tags, tags = self.update_tags(response['tags'])
                    if update_tags:
                        changed = True
                        self.tags = tags
                if self.public_network_access is not None and self.public_network_access != response['public_network_access']:
                    changed = True
                else:
                    self.public_network_access = response['public_network_access']

                if self.repo_configuration is not None and self.repo_configuration != response['repo_configuration']:
                    changed = True
                else:
                    self.repo_configuration = response['repo_configuration']
            else:
                changed = True

            if self.check_mode:
                changed = True
                self.log("Check mode test, Data factory will be create or update")
            else:
                if changed:
                    if self.repo_configuration:
                        if self.repo_configuration['type'] == 'FactoryGitHubConfiguration':
                            repo_parameters = self.datafactory_model.FactoryGitHubConfiguration(
                                account_name=self.repo_configuration.get('account_name'),
                                repository_name=self.repo_configuration.get('repository_name'),
                                collaboration_branch=self.repo_configuration.get('collaboration_branch'),
                                root_folder=self.repo_configuration.get('root_folder')
                            )
                        else:
                            repo_parameters = self.datafactory_model.FactoryVSTSConfiguration(
                                account_name=self.repo_configuration.get('account_name'),
                                repository_name=self.repo_configuration.get('repository_name'),
                                collaboration_branch=self.repo_configuration.get('collaboration_branch'),
                                root_folder=self.repo_configuration.get('root_folder'),
                                project_name=self.repo_configuration.get('project_name'),
                            )
                    else:
                        repo_parameters = None

                    update_parameters = self.datafactory_model.Factory(
                        location=self.location,
                        tags=self.tags,
                        public_network_access=self.public_network_access,
                        repo_configuration=repo_parameters
                    )

                    response = self.create_or_update(update_parameters)

        else:
            if self.check_mode:
                changed = True
                self.log("Check mode test")
            if response:
                self.log("The Data factory {0} exist, will be deleted".format(self.name))
                changed = True
                response = self.delete()
            else:
                changed = False

        self.results['changed'] = changed
        self.results['state'] = response
        return self.results

    def get_item(self):
        response = None
        self.log('Get properties for {0}'.format(self.name))
        try:
            response = self.datafactory_client.factories.get(self.resource_group, self.name)
        except ResourceNotFoundError:
            pass
        return self.pip_to_dict(response) if response else None

    def delete(self):
        response = None
        self.log('Delete data factory for {0}'.format(self.name))
        try:
            response = self.datafactory_client.factories.delete(self.resource_group, self.name)
        except Exception as ec:
            self.fail("Delete fail {0}, error message {1}".format(self.name, ec))
        return self.pip_to_dict(response) if response else None

    def create_or_update(self, parameters):
        response = None
        self.log('Create data factory for {0}'.format(self.name))
        try:
            response = self.datafactory_client.factories.create_or_update(self.resource_group,
                                                                          self.name,
                                                                          parameters,
                                                                          self.if_match)
        except Exception as ec:
            self.fail("Create fail {0}, error message {1}".format(self.name, ec))
        return self.pip_to_dict(response) if response else None

    def pip_to_dict(self, pip):
        result = dict(
            id=pip.id,
            name=pip.name,
            type=pip.type,
            location=pip.location,
            tags=pip.tags,
            e_tag=pip.e_tag,
            provisioning_state=pip.provisioning_state,
            create_time=pip.create_time,
            public_network_access=pip.public_network_access,
            repo_configuration=dict(),
            identity=dict()
        )
        if pip.identity:
            result['identity']['principal_id'] = pip.identity.principal_id
            result['identity']['tenant_id'] = pip.identity.tenant_id
        if pip.repo_configuration:
            result['repo_configuration']['account_name'] = pip.repo_configuration.account_name
            result['repo_configuration']['repository_name'] = pip.repo_configuration.repository_name
            result['repo_configuration']['collaboration_branch'] = pip.repo_configuration.collaboration_branch
            result['repo_configuration']['root_folder'] = pip.repo_configuration.root_folder
            result['repo_configuration']['type'] = pip.repo_configuration.type
            if pip.repo_configuration.type == "FactoryVSTSConfiguration":
                result['repo_configuration']['project_name'] = pip.repo_configuration.project_name
        return result


def main():
    AzureRMDataFactory()


if __name__ == '__main__':
    main()
